# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'icaDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(517, 647)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.bottomLayout = QtWidgets.QHBoxLayout()
        self.bottomLayout.setObjectName("bottomLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.bottomLayout.addItem(spacerItem)
        self.pushButtonCancel = QtWidgets.QPushButton(Dialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.bottomLayout.addWidget(self.pushButtonCancel)
        self.pushButtonApply = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonApply.sizePolicy().hasHeightForWidth())
        self.pushButtonApply.setSizePolicy(sizePolicy)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.bottomLayout.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.bottomLayout, 1, 0, 1, 1)
        self.scrollArea = QtWidgets.QScrollArea(Dialog)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 499, 587))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.groupBoxComputation = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxComputation.setObjectName("groupBoxComputation")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBoxComputation)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.parametersLayout = QtWidgets.QGridLayout()
        self.parametersLayout.setObjectName("parametersLayout")
        self.spinBoxMaxIter = QtWidgets.QSpinBox(self.groupBoxComputation)
        self.spinBoxMaxIter.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.spinBoxMaxIter.setMinimum(1)
        self.spinBoxMaxIter.setMaximum(10000000)
        self.spinBoxMaxIter.setProperty("value", 2000)
        self.spinBoxMaxIter.setObjectName("spinBoxMaxIter")
        self.parametersLayout.addWidget(self.spinBoxMaxIter, 1, 1, 1, 1)
        self.labelNComponents = QtWidgets.QLabel(self.groupBoxComputation)
        self.labelNComponents.setObjectName("labelNComponents")
        self.parametersLayout.addWidget(self.labelNComponents, 0, 0, 1, 1)
        self.doubleSpinBoxNComponents = QtWidgets.QDoubleSpinBox(self.groupBoxComputation)
        self.doubleSpinBoxNComponents.setSuffix("")
        self.doubleSpinBoxNComponents.setDecimals(3)
        self.doubleSpinBoxNComponents.setMaximum(1000.0)
        self.doubleSpinBoxNComponents.setSingleStep(0.1)
        self.doubleSpinBoxNComponents.setProperty("value", 0.95)
        self.doubleSpinBoxNComponents.setObjectName("doubleSpinBoxNComponents")
        self.parametersLayout.addWidget(self.doubleSpinBoxNComponents, 0, 1, 1, 1)
        self.labelMaxIter = QtWidgets.QLabel(self.groupBoxComputation)
        self.labelMaxIter.setScaledContents(False)
        self.labelMaxIter.setObjectName("labelMaxIter")
        self.parametersLayout.addWidget(self.labelMaxIter, 1, 0, 1, 1)
        self.gridLayout_4.addLayout(self.parametersLayout, 0, 0, 1, 1)
        self.pushButtonCompute = QtWidgets.QPushButton(self.groupBoxComputation)
        self.pushButtonCompute.setObjectName("pushButtonCompute")
        self.gridLayout_4.addWidget(self.pushButtonCompute, 1, 0, 1, 1)
        self.gridLayout_5.addWidget(self.groupBoxComputation, 0, 0, 1, 1)
        self.groupBoxComponents = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxComponents.setObjectName("groupBoxComponents")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBoxComponents)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.labelNotRemoved = QtWidgets.QLabel(self.groupBoxComponents)
        self.labelNotRemoved.setObjectName("labelNotRemoved")
        self.gridLayout_2.addWidget(self.labelNotRemoved, 0, 0, 1, 1)
        self.listWidgetNotRemoved = QtWidgets.QListWidget(self.groupBoxComponents)
        self.listWidgetNotRemoved.setObjectName("listWidgetNotRemoved")
        self.gridLayout_2.addWidget(self.listWidgetNotRemoved, 1, 0, 1, 1)
        self.listWidgetRemoved = QtWidgets.QListWidget(self.groupBoxComponents)
        self.listWidgetRemoved.setObjectName("listWidgetRemoved")
        self.gridLayout_2.addWidget(self.listWidgetRemoved, 4, 0, 1, 1)
        self.labelRemoved = QtWidgets.QLabel(self.groupBoxComponents)
        self.labelRemoved.setObjectName("labelRemoved")
        self.gridLayout_2.addWidget(self.labelRemoved, 3, 0, 1, 1)
        self.pushButtonTransfer = QtWidgets.QPushButton(self.groupBoxComponents)
        self.pushButtonTransfer.setObjectName("pushButtonTransfer")
        self.gridLayout_2.addWidget(self.pushButtonTransfer, 2, 0, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout_2)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.pushButtonPlotProperties = QtWidgets.QPushButton(self.groupBoxComponents)
        self.pushButtonPlotProperties.setObjectName("pushButtonPlotProperties")
        self.gridLayout_3.addWidget(self.pushButtonPlotProperties, 2, 0, 1, 1)
        self.pushButtonPlotChanges = QtWidgets.QPushButton(self.groupBoxComponents)
        self.pushButtonPlotChanges.setObjectName("pushButtonPlotChanges")
        self.gridLayout_3.addWidget(self.pushButtonPlotChanges, 3, 0, 1, 1)
        self.pushButtonPlotSources = QtWidgets.QPushButton(self.groupBoxComponents)
        self.pushButtonPlotSources.setObjectName("pushButtonPlotSources")
        self.gridLayout_3.addWidget(self.pushButtonPlotSources, 0, 0, 1, 1)
        self.pushButtonPlotTopographies = QtWidgets.QPushButton(self.groupBoxComponents)
        self.pushButtonPlotTopographies.setObjectName("pushButtonPlotTopographies")
        self.gridLayout_3.addWidget(self.pushButtonPlotTopographies, 1, 0, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout_3)
        self.gridLayout_7.addLayout(self.horizontalLayout, 0, 0, 1, 1)
        self.gridLayout_5.addWidget(self.groupBoxComponents, 1, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.pushButtonCancel.clicked.connect(Dialog.reject)
        self.pushButtonApply.clicked.connect(Dialog.accept)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Meggie - ICA"))
        self.pushButtonCancel.setText(_translate("Dialog", "Cancel"))
        self.pushButtonApply.setText(_translate("Dialog", "Apply"))
        self.groupBoxComputation.setTitle(_translate("Dialog", "ICA computation"))
        self.labelNComponents.setText(_translate("Dialog", "Explained variance:"))
        self.labelMaxIter.setText(_translate("Dialog", "Maximum number of iterations: "))
        self.pushButtonCompute.setText(_translate("Dialog", "Compute"))
        self.groupBoxComponents.setTitle(_translate("Dialog", "ICA components"))
        self.labelNotRemoved.setText(_translate("Dialog", "Not removed"))
        self.labelRemoved.setText(_translate("Dialog", "Removed"))
        self.pushButtonTransfer.setText(_translate("Dialog", "Transfer"))
        self.pushButtonPlotProperties.setText(_translate("Dialog", "Plot properties"))
        self.pushButtonPlotChanges.setText(_translate("Dialog", "Plot changes"))
        self.pushButtonPlotSources.setText(_translate("Dialog", "Plot time courses"))
        self.pushButtonPlotTopographies.setText(_translate("Dialog", "Plot topographies"))




""" Contains threading related utilities.
"""

from queue import Queue
from queue import Empty
from time import sleep
from sys import exc_info
from multiprocessing.pool import ThreadPool

from PyQt5 import QtGui
from PyQt5 import QtCore
from PyQt5 import QtWidgets


def threaded(func):
    """Allows decorating functions so that they will be run in a separate thread. 

    This is not meant for parallelization but to keep the Qt window 
    responsive while doing a resource intensive task.

    Carefully pipes the exceptions out of the thread to the main thread.

    Parameters
    ----------
    func : function
        The function to be wrapped.

    Returns
    -------
    function 
        The wrapped function.
    """
    def decorated(*args, **kwargs):
        """ Inner function for threaded-decoration """

        # allow bypassing threads for testing
        if kwargs.pop('no_threading', None):
            return func(*args, **kwargs)

        # worker threads should be used on time consuming
        # tasks so add a indicator for user
        QtWidgets.QApplication.setOverrideCursor(
            QtGui.QCursor(QtCore.Qt.WaitCursor))

        pool = ThreadPool(processes=1)
        do_meanwhile = kwargs.pop('do_meanwhile', None)
        bucket = Queue()

        # exceptions are carried over from worker thread
        # to main thread
        def exception_wrapper():
            """ Helper to get exception info out of thread """
            try:
                result = func(*args, **kwargs)
            except BaseException:
                bucket.put(exc_info())
            return result

        async_result = pool.apply_async(exception_wrapper)
        while True:
            try:
                exc = bucket.get(block=False)
                pool.terminate()
                QtWidgets.QApplication.restoreOverrideCursor()

                raise exc[1].with_traceback(exc[2])

            except Empty:
                pass
            if async_result.ready():
                break
            if do_meanwhile:
                do_meanwhile()
            sleep(0.2)
        result = async_result.get()
        pool.terminate()

        # everything went fine and control should return to user
        QtWidgets.QApplication.restoreOverrideCursor()

        return result
    return decorated




import os

import matplotlib.pyplot as plt

from meggie.utilities.names import next_available_name


def test_next_available_name():
    names = ["kissa", "koira", "kissa_1", "kissa_2", "kissa_11", "kissa_kala"]
    assert(next_available_name(names, "kissa") == 'kissa_12')
    assert(next_available_name(names, "koira") == 'koira_1')
    assert(next_available_name(names, "kettu") == 'kettu')
    assert(next_available_name(names, "kissa_kala") == 'kissa_kala_1')

    names = ['EO', 'EOEC', 'group_EOEC']
    assert(next_available_name(names, 'group_EO') == 'group_EO')




""" Contains save evoked action handling.
"""

from meggie.utilities.messaging import exc_messagebox
from meggie.utilities.messaging import messagebox
from meggie.utilities.channels import get_channels_by_type
from meggie.utilities.validators import assert_arrays_same

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action

from meggie.actions.evoked_save.controller.evoked import save_channel_averages
from meggie.actions.evoked_save.controller.evoked import save_all_channels

from meggie.utilities.dialogs.outputOptionsMain import OutputOptions


class SaveEvoked(Action):
    """ Saved evoked items to csv files """

    def run(self):
        try:
            selected_name = self.data['outputs']['evoked'][0]
        except IndexError as exc:
            return

        # validate times
        time_arrays = []
        for subject in self.experiment.subjects.values():
            evoked = subject.evoked.get(selected_name)
            if not evoked:
                continue
            for mne_evoked in evoked.content.values():
                time_arrays.append(mne_evoked.times)

        assert_arrays_same(time_arrays, 'Times do not match')

        def option_handler(selected_option):
            params = {'name': selected_name, 
                      'output_option': selected_option,
                      'channel_groups': self.experiment.channel_groups}
            try:
                self.handler(self.experiment.active_subject, params)
            except Exception as exc:
                exc_messagebox(self.window, exc)

        dialog = OutputOptions(self.window, handler=option_handler)
        dialog.show()

    @subject_action
    def handler(self, subject, params):
        """
        """
        if params['output_option'] == 'channel_averages':
            save_channel_averages(self.experiment, 
                                  params['name'], 
                                  params['channel_groups'],
                                  do_meanwhile=self.window.update_ui)
        else:
            save_all_channels(self.experiment, params['name'],
                              do_meanwhile=self.window.update_ui)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'powerSpectrumDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PowerSpectrumDialog(object):
    def setupUi(self, PowerSpectrumDialog):
        PowerSpectrumDialog.setObjectName("PowerSpectrumDialog")
        PowerSpectrumDialog.resize(561, 712)
        self.gridLayout = QtWidgets.QGridLayout(PowerSpectrumDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(PowerSpectrumDialog)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 527, 840))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.groupBoxGeneral = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxGeneral.setObjectName("groupBoxGeneral")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBoxGeneral)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.labelName = QtWidgets.QLabel(self.groupBoxGeneral)
        self.labelName.setObjectName("labelName")
        self.gridLayout_3.addWidget(self.labelName, 0, 0, 1, 1)
        self.lineEditName = QtWidgets.QLineEdit(self.groupBoxGeneral)
        self.lineEditName.setObjectName("lineEditName")
        self.gridLayout_3.addWidget(self.lineEditName, 0, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 0, 1, 1, 1)
        self.gridLayout_4.addWidget(self.groupBoxGeneral, 0, 0, 1, 1)
        self.groupBoxIntervals = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBoxIntervals.sizePolicy().hasHeightForWidth())
        self.groupBoxIntervals.setSizePolicy(sizePolicy)
        self.groupBoxIntervals.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBoxIntervals.setObjectName("groupBoxIntervals")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBoxIntervals)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.labelGroup = QtWidgets.QLabel(self.groupBoxIntervals)
        self.labelGroup.setObjectName("labelGroup")
        self.gridLayout_2.addWidget(self.labelGroup, 0, 0, 1, 1)
        self.labelTmax = QtWidgets.QLabel(self.groupBoxIntervals)
        self.labelTmax.setObjectName("labelTmax")
        self.gridLayout_2.addWidget(self.labelTmax, 2, 0, 1, 1)
        self.doubleSpinBoxTmin = QtWidgets.QDoubleSpinBox(self.groupBoxIntervals)
        self.doubleSpinBoxTmin.setMaximum(1000000000.0)
        self.doubleSpinBoxTmin.setObjectName("doubleSpinBoxTmin")
        self.gridLayout_2.addWidget(self.doubleSpinBoxTmin, 1, 1, 1, 1)
        self.pushButtonAdd = QtWidgets.QPushButton(self.groupBoxIntervals)
        self.pushButtonAdd.setObjectName("pushButtonAdd")
        self.gridLayout_2.addWidget(self.pushButtonAdd, 3, 0, 1, 2)
        self.comboBoxAvgGroup = QtWidgets.QComboBox(self.groupBoxIntervals)
        self.comboBoxAvgGroup.setObjectName("comboBoxAvgGroup")
        self.comboBoxAvgGroup.addItem("")
        self.comboBoxAvgGroup.addItem("")
        self.comboBoxAvgGroup.addItem("")
        self.comboBoxAvgGroup.addItem("")
        self.comboBoxAvgGroup.addItem("")
        self.comboBoxAvgGroup.addItem("")
        self.comboBoxAvgGroup.addItem("")
        self.comboBoxAvgGroup.addItem("")
        self.gridLayout_2.addWidget(self.comboBoxAvgGroup, 0, 1, 1, 1)
        self.doubleSpinBoxTmax = QtWidgets.QDoubleSpinBox(self.groupBoxIntervals)
        self.doubleSpinBoxTmax.setMaximum(1000000000.0)
        self.doubleSpinBoxTmax.setObjectName("doubleSpinBoxTmax")
        self.gridLayout_2.addWidget(self.doubleSpinBoxTmax, 2, 1, 1, 1)
        self.labelTmin = QtWidgets.QLabel(self.groupBoxIntervals)
        self.labelTmin.setObjectName("labelTmin")
        self.gridLayout_2.addWidget(self.labelTmin, 1, 0, 1, 1)
        self.pushButtonAddAdvanced = QtWidgets.QPushButton(self.groupBoxIntervals)
        self.pushButtonAddAdvanced.setObjectName("pushButtonAddAdvanced")
        self.gridLayout_2.addWidget(self.pushButtonAddAdvanced, 4, 0, 1, 2)
        self.pushButtonClearRow = QtWidgets.QPushButton(self.groupBoxIntervals)
        self.pushButtonClearRow.setObjectName("pushButtonClearRow")
        self.gridLayout_2.addWidget(self.pushButtonClearRow, 5, 0, 1, 2)
        self.pushButtonClear = QtWidgets.QPushButton(self.groupBoxIntervals)
        self.pushButtonClear.setObjectName("pushButtonClear")
        self.gridLayout_2.addWidget(self.pushButtonClear, 6, 0, 1, 2)
        self.listWidgetIntervals = QtWidgets.QListWidget(self.groupBoxIntervals)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidgetIntervals.sizePolicy().hasHeightForWidth())
        self.listWidgetIntervals.setSizePolicy(sizePolicy)
        self.listWidgetIntervals.setObjectName("listWidgetIntervals")
        self.gridLayout_2.addWidget(self.listWidgetIntervals, 0, 2, 7, 1)
        self.gridLayout_4.addWidget(self.groupBoxIntervals, 2, 0, 1, 1)
        self.groupBoxConditions = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxConditions.setObjectName("groupBoxConditions")
        self.formLayout = QtWidgets.QFormLayout(self.groupBoxConditions)
        self.formLayout.setObjectName("formLayout")
        self.labelFmin = QtWidgets.QLabel(self.groupBoxConditions)
        self.labelFmin.setObjectName("labelFmin")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.labelFmin)
        self.spinBoxFmin = QtWidgets.QSpinBox(self.groupBoxConditions)
        self.spinBoxFmin.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.spinBoxFmin.setMaximum(1000)
        self.spinBoxFmin.setProperty("value", 1)
        self.spinBoxFmin.setObjectName("spinBoxFmin")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.spinBoxFmin)
        self.labelFmax = QtWidgets.QLabel(self.groupBoxConditions)
        self.labelFmax.setObjectName("labelFmax")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.labelFmax)
        self.spinBoxFmax = QtWidgets.QSpinBox(self.groupBoxConditions)
        self.spinBoxFmax.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.spinBoxFmax.setMaximum(1000)
        self.spinBoxFmax.setProperty("value", 40)
        self.spinBoxFmax.setObjectName("spinBoxFmax")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.spinBoxFmax)
        self.labelOverlap = QtWidgets.QLabel(self.groupBoxConditions)
        self.labelOverlap.setObjectName("labelOverlap")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.labelOverlap)
        self.spinBoxOverlap = QtWidgets.QSpinBox(self.groupBoxConditions)
        self.spinBoxOverlap.setMaximum(100000)
        self.spinBoxOverlap.setProperty("value", 256)
        self.spinBoxOverlap.setObjectName("spinBoxOverlap")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.spinBoxOverlap)
        self.labelNfft = QtWidgets.QLabel(self.groupBoxConditions)
        self.labelNfft.setObjectName("labelNfft")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.labelNfft)
        self.spinBoxNfft = QtWidgets.QSpinBox(self.groupBoxConditions)
        self.spinBoxNfft.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.spinBoxNfft.setMaximum(100000)
        self.spinBoxNfft.setProperty("value", 512)
        self.spinBoxNfft.setObjectName("spinBoxNfft")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.spinBoxNfft)
        self.gridLayout_4.addWidget(self.groupBoxConditions, 1, 0, 1, 1)
        self.groupBoxBatching = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBatching.setObjectName("groupBoxBatching")
        self.gridLayoutBatching = QtWidgets.QGridLayout(self.groupBoxBatching)
        self.gridLayoutBatching.setObjectName("gridLayoutBatching")
        self.batchingWidgetPlaceholder = QtWidgets.QWidget(self.groupBoxBatching)
        self.batchingWidgetPlaceholder.setMinimumSize(QtCore.QSize(300, 300))
        self.batchingWidgetPlaceholder.setObjectName("batchingWidgetPlaceholder")
        self.gridLayoutBatching.addWidget(self.batchingWidgetPlaceholder, 0, 0, 1, 1)
        self.gridLayout_4.addWidget(self.groupBoxBatching, 3, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)
        self.horizontalLayoutButtons = QtWidgets.QHBoxLayout()
        self.horizontalLayoutButtons.setObjectName("horizontalLayoutButtons")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayoutButtons.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(PowerSpectrumDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayoutButtons.addWidget(self.pushButtonCancel)
        self.pushButtonBatch = QtWidgets.QPushButton(PowerSpectrumDialog)
        self.pushButtonBatch.setObjectName("pushButtonBatch")
        self.horizontalLayoutButtons.addWidget(self.pushButtonBatch)
        self.pushButtonApply = QtWidgets.QPushButton(PowerSpectrumDialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayoutButtons.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.horizontalLayoutButtons, 1, 0, 1, 1)

        self.retranslateUi(PowerSpectrumDialog)
        self.pushButtonApply.clicked.connect(PowerSpectrumDialog.accept)
        self.pushButtonCancel.clicked.connect(PowerSpectrumDialog.reject)
        self.pushButtonBatch.clicked.connect(PowerSpectrumDialog.acceptBatch)
        QtCore.QMetaObject.connectSlotsByName(PowerSpectrumDialog)

    def retranslateUi(self, PowerSpectrumDialog):
        _translate = QtCore.QCoreApplication.translate
        PowerSpectrumDialog.setWindowTitle(_translate("PowerSpectrumDialog", "Meggie - Create spectrums"))
        self.groupBoxGeneral.setTitle(_translate("PowerSpectrumDialog", "General"))
        self.labelName.setText(_translate("PowerSpectrumDialog", "Spectrum name:"))
        self.groupBoxIntervals.setTitle(_translate("PowerSpectrumDialog", "Select time intervals"))
        self.labelGroup.setText(_translate("PowerSpectrumDialog", "Average group:"))
        self.labelTmax.setText(_translate("PowerSpectrumDialog", "End time:"))
        self.doubleSpinBoxTmin.setSuffix(_translate("PowerSpectrumDialog", " s"))
        self.pushButtonAdd.setText(_translate("PowerSpectrumDialog", "Add to list"))
        self.comboBoxAvgGroup.setItemText(0, _translate("PowerSpectrumDialog", "1"))
        self.comboBoxAvgGroup.setItemText(1, _translate("PowerSpectrumDialog", "2"))
        self.comboBoxAvgGroup.setItemText(2, _translate("PowerSpectrumDialog", "3"))
        self.comboBoxAvgGroup.setItemText(3, _translate("PowerSpectrumDialog", "4"))
        self.comboBoxAvgGroup.setItemText(4, _translate("PowerSpectrumDialog", "5"))
        self.comboBoxAvgGroup.setItemText(5, _translate("PowerSpectrumDialog", "6"))
        self.comboBoxAvgGroup.setItemText(6, _translate("PowerSpectrumDialog", "7"))
        self.comboBoxAvgGroup.setItemText(7, _translate("PowerSpectrumDialog", "8"))
        self.doubleSpinBoxTmax.setSuffix(_translate("PowerSpectrumDialog", " s"))
        self.labelTmin.setText(_translate("PowerSpectrumDialog", "Start time:"))
        self.pushButtonAddAdvanced.setText(_translate("PowerSpectrumDialog", "Add advanced..."))
        self.pushButtonClearRow.setText(_translate("PowerSpectrumDialog", "Clear row"))
        self.pushButtonClear.setText(_translate("PowerSpectrumDialog", "Clear list"))
        self.groupBoxConditions.setTitle(_translate("PowerSpectrumDialog", "Conditions"))
        self.labelFmin.setText(_translate("PowerSpectrumDialog", "Min frequency of interest:"))
        self.spinBoxFmin.setSuffix(_translate("PowerSpectrumDialog", "Hz"))
        self.labelFmax.setText(_translate("PowerSpectrumDialog", "Max frequency of interest:"))
        self.spinBoxFmax.setSuffix(_translate("PowerSpectrumDialog", "Hz"))
        self.labelOverlap.setText(_translate("PowerSpectrumDialog", "Overlap:"))
        self.labelNfft.setText(_translate("PowerSpectrumDialog", "Length of window:"))
        self.groupBoxBatching.setTitle(_translate("PowerSpectrumDialog", "Batching"))
        self.pushButtonCancel.setText(_translate("PowerSpectrumDialog", "Cancel"))
        self.pushButtonBatch.setText(_translate("PowerSpectrumDialog", "Batch"))
        self.pushButtonApply.setText(_translate("PowerSpectrumDialog", "Apply"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'montageDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_montageDialog(object):
    def setupUi(self, montageDialog):
        montageDialog.setObjectName("montageDialog")
        montageDialog.resize(403, 557)
        self.gridLayout = QtWidgets.QGridLayout(montageDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(montageDialog)
        self.scrollArea.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 369, 517))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 3, 1, 1, 1)
        self.groupBoxMontage = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxMontage.setObjectName("groupBoxMontage")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBoxMontage)
        self.formLayout_2.setObjectName("formLayout_2")
        self.radioButtonMontageFromList = QtWidgets.QRadioButton(self.groupBoxMontage)
        self.radioButtonMontageFromList.setObjectName("radioButtonMontageFromList")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.radioButtonMontageFromList)
        self.comboBoxSelectFromList = QtWidgets.QComboBox(self.groupBoxMontage)
        self.comboBoxSelectFromList.setObjectName("comboBoxSelectFromList")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.comboBoxSelectFromList)
        self.radioButtonMontageFromFile = QtWidgets.QRadioButton(self.groupBoxMontage)
        self.radioButtonMontageFromFile.setObjectName("radioButtonMontageFromFile")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.radioButtonMontageFromFile)
        self.pushButtonSelectFromFile = QtWidgets.QPushButton(self.groupBoxMontage)
        self.pushButtonSelectFromFile.setObjectName("pushButtonSelectFromFile")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.pushButtonSelectFromFile)
        self.labelCurrentLabel = QtWidgets.QLabel(self.groupBoxMontage)
        self.labelCurrentLabel.setObjectName("labelCurrentLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.labelCurrentLabel)
        self.labelCurrentContent = QtWidgets.QLabel(self.groupBoxMontage)
        self.labelCurrentContent.setText("")
        self.labelCurrentContent.setObjectName("labelCurrentContent")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.labelCurrentContent)
        self.labelHeadSize = QtWidgets.QLabel(self.groupBoxMontage)
        self.labelHeadSize.setObjectName("labelHeadSize")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.labelHeadSize)
        self.doubleSpinBoxHeadSize = QtWidgets.QDoubleSpinBox(self.groupBoxMontage)
        self.doubleSpinBoxHeadSize.setDecimals(4)
        self.doubleSpinBoxHeadSize.setMaximum(1.0)
        self.doubleSpinBoxHeadSize.setSingleStep(0.001)
        self.doubleSpinBoxHeadSize.setProperty("value", 0.095)
        self.doubleSpinBoxHeadSize.setObjectName("doubleSpinBoxHeadSize")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxHeadSize)
        self.gridLayout_2.addWidget(self.groupBoxMontage, 0, 0, 2, 2)
        self.groupBoxBatching = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBatching.setObjectName("groupBoxBatching")
        self.gridLayoutBatching = QtWidgets.QGridLayout(self.groupBoxBatching)
        self.gridLayoutBatching.setObjectName("gridLayoutBatching")
        self.batchingWidgetPlaceholder = QtWidgets.QWidget(self.groupBoxBatching)
        self.batchingWidgetPlaceholder.setMinimumSize(QtCore.QSize(300, 300))
        self.batchingWidgetPlaceholder.setObjectName("batchingWidgetPlaceholder")
        self.gridLayoutBatching.addWidget(self.batchingWidgetPlaceholder, 0, 0, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxBatching, 2, 0, 1, 2)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(montageDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonBatch = QtWidgets.QPushButton(montageDialog)
        self.pushButtonBatch.setObjectName("pushButtonBatch")
        self.horizontalLayout.addWidget(self.pushButtonBatch)
        self.pushButtonApply = QtWidgets.QPushButton(montageDialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayout.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(montageDialog)
        self.pushButtonCancel.clicked.connect(montageDialog.reject)
        self.pushButtonApply.clicked.connect(montageDialog.accept)
        self.pushButtonBatch.clicked.connect(montageDialog.acceptBatch)
        QtCore.QMetaObject.connectSlotsByName(montageDialog)
        montageDialog.setTabOrder(self.pushButtonCancel, self.pushButtonBatch)
        montageDialog.setTabOrder(self.pushButtonBatch, self.pushButtonApply)
        montageDialog.setTabOrder(self.pushButtonApply, self.scrollArea)
        montageDialog.setTabOrder(self.scrollArea, self.radioButtonMontageFromList)
        montageDialog.setTabOrder(self.radioButtonMontageFromList, self.comboBoxSelectFromList)
        montageDialog.setTabOrder(self.comboBoxSelectFromList, self.radioButtonMontageFromFile)
        montageDialog.setTabOrder(self.radioButtonMontageFromFile, self.pushButtonSelectFromFile)

    def retranslateUi(self, montageDialog):
        _translate = QtCore.QCoreApplication.translate
        montageDialog.setWindowTitle(_translate("montageDialog", "Meggie - Set standard montage"))
        self.groupBoxMontage.setTitle(_translate("montageDialog", "Select EEG montage:"))
        self.radioButtonMontageFromList.setText(_translate("montageDialog", "Select from list:"))
        self.radioButtonMontageFromFile.setText(_translate("montageDialog", "Select from file:"))
        self.pushButtonSelectFromFile.setText(_translate("montageDialog", "Browse..."))
        self.labelCurrentLabel.setText(_translate("montageDialog", "Current selection:"))
        self.labelHeadSize.setText(_translate("montageDialog", "Head size:"))
        self.doubleSpinBoxHeadSize.setSuffix(_translate("montageDialog", "m"))
        self.groupBoxBatching.setTitle(_translate("montageDialog", "Batching"))
        self.pushButtonCancel.setText(_translate("montageDialog", "Cancel"))
        self.pushButtonBatch.setText(_translate("montageDialog", "Batch"))
        self.pushButtonApply.setText(_translate("montageDialog", "Apply"))




import tempfile
import os

import mne
from mne.time_frequency import psd_welch

from meggie.datatypes.spectrum.spectrum import Spectrum

from meggie.utilities.events import find_events
from meggie.utilities.filemanager import ensure_folders

def test_spectrum():
    with tempfile.TemporaryDirectory() as dirpath:

        sample_folder = mne.datasets.sample.data_path()
        sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')

        raw = mne.io.read_raw_fif(sample_fname, preload=True)
        psds, freqs = psd_welch(raw, fmin=1, fmax=40, tmin=1, tmax=10)
        ch_names = raw.info['ch_names']

        name = 'TestSpectrum'
        cond_name = '1'
        spectrum_dir = os.path.join(dirpath, 'spectrums')

        # one meggie-Spectrum can hold many spectrums, thus content is dict-like
        content = {cond_name: psds}
        params = {'conditions': [cond_name]}

        # Create meggie-Spectrum object with spectrum array stored within
        # and save it to spectrum directory
        spectrum = Spectrum(name, spectrum_dir, params, 
                            content=content, freqs=freqs, info=raw.info)
        ensure_folders([spectrum_dir])
        spectrum.save_content()

        # Creating meggie-Spectrum object with same name and folder should allow
        # accessing the saved content
        loaded_spectrum = Spectrum(name, spectrum_dir, params)

        assert(list(loaded_spectrum.content.keys())[0] == cond_name)




""" Helpers to handle data units. Data in memory is in standard units, i.e
    for magnetometers it is T, gradiometers it is T/m, and electrodes it is V.
    However it is customary to convert these into fT, fT/cm and uV respectively
    for human reading. For power densities it follows that the units will be
    T^2/Hz, (T/m)^2/Hz and V^2/Hz. This is because density is the FT of the
    autocorrelation of the signal, and thus if the original unit is X,
    the resulting unit is X^2/Hz. Or if the logarithmic transformation is done,
    unit is dB/Hz for each.)
"""

UNITS = {
    'mag': {
        'scaling': 1e15,
        'unit': 'fT',
        'power_unit': 'T^2/Hz',
    },
    'grad': {
        'scaling': 1e13,
        'unit': 'fT/cm',
        'power_unit': '(T/m)^2/Hz',
    },
    'eeg': {
        'scaling': 1e6,
        'unit': 'uV',
        'power_unit': 'V^2/Hz'
    },
    'eog': {
        'scaling': 1e6,
        'unit': 'uV',
        'power_unit': 'V^2/Hz'
    },

    'ecg': {
        'scaling': 1e6,
        'unit': 'uV',
        'power_unit': 'V^2/Hz'
    }
}


def get_scaling(type_):
    """Returns standard scaling factor for a sensor type.

    Parameters
    ----------
    type_ : str
        The sensor type.

    Returns
    -------
    float
        The scaling factor.
    """
    if type_ not in UNITS:
        raise Exception('Unknown data type')
    return UNITS[type_]['scaling']


def get_unit(type_):
    """Returns unit of signal for a sensor type.

    Parameters
    ----------
    type_ : str
        The sensor type.

    Returns
    -------
    str
        The unit.
    """
    if type_ not in UNITS:
        raise Exception('Unknown data type')
    return UNITS[type_]['unit']


def get_power_unit(type_, log=False):
    """Returns unit of power for a sensor type.

    Parameters
    ----------
    type_ : str
        The sensor type.
    log : bool
        Whether the unit is for log transformed data.

    Returns
    -------
    str
        The unit.
    """
    if type_ not in UNITS:
        raise Exception('Unknown data type')
    if log:
        return 'dB/Hz'
    else:
        return UNITS[type_]['power_unit']



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'channelGroupsDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_channelGroupsDialog(object):
    def setupUi(self, channelGroupsDialog):
        channelGroupsDialog.setObjectName("channelGroupsDialog")
        channelGroupsDialog.resize(398, 324)
        self.gridLayout = QtWidgets.QGridLayout(channelGroupsDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(channelGroupsDialog)
        self.scrollArea.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 378, 273))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBoxChannelGroups = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxChannelGroups.setObjectName("groupBoxChannelGroups")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBoxChannelGroups)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.pushButtonSetChannels = QtWidgets.QPushButton(self.groupBoxChannelGroups)
        self.pushButtonSetChannels.setObjectName("pushButtonSetChannels")
        self.gridLayout_3.addWidget(self.pushButtonSetChannels, 4, 1, 1, 1)
        self.pushButtonAdd = QtWidgets.QPushButton(self.groupBoxChannelGroups)
        self.pushButtonAdd.setObjectName("pushButtonAdd")
        self.gridLayout_3.addWidget(self.pushButtonAdd, 0, 1, 1, 1)
        self.listWidgetChannelGroups = QtWidgets.QListWidget(self.groupBoxChannelGroups)
        self.listWidgetChannelGroups.setObjectName("listWidgetChannelGroups")
        self.gridLayout_3.addWidget(self.listWidgetChannelGroups, 2, 0, 3, 1)
        self.pushButtonRemove = QtWidgets.QPushButton(self.groupBoxChannelGroups)
        self.pushButtonRemove.setObjectName("pushButtonRemove")
        self.gridLayout_3.addWidget(self.pushButtonRemove, 2, 1, 1, 1)
        self.lineEditAdd = QtWidgets.QLineEdit(self.groupBoxChannelGroups)
        self.lineEditAdd.setObjectName("lineEditAdd")
        self.gridLayout_3.addWidget(self.lineEditAdd, 0, 0, 1, 1)
        self.pushButtonReset = QtWidgets.QPushButton(self.groupBoxChannelGroups)
        self.pushButtonReset.setObjectName("pushButtonReset")
        self.gridLayout_3.addWidget(self.pushButtonReset, 3, 1, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxChannelGroups, 1, 0, 1, 1)
        self.groupBoxChannelType = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxChannelType.setObjectName("groupBoxChannelType")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBoxChannelType)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.radioButtonMEG = QtWidgets.QRadioButton(self.groupBoxChannelType)
        self.radioButtonMEG.setChecked(True)
        self.radioButtonMEG.setObjectName("radioButtonMEG")
        self.horizontalLayout_2.addWidget(self.radioButtonMEG)
        self.radioButtonEEG = QtWidgets.QRadioButton(self.groupBoxChannelType)
        self.radioButtonEEG.setObjectName("radioButtonEEG")
        self.horizontalLayout_2.addWidget(self.radioButtonEEG)
        self.gridLayout_5.addLayout(self.horizontalLayout_2, 0, 0, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxChannelType, 0, 0, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 2, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(channelGroupsDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonApply = QtWidgets.QPushButton(channelGroupsDialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayout.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(channelGroupsDialog)
        self.pushButtonCancel.clicked.connect(channelGroupsDialog.reject)
        self.pushButtonApply.clicked.connect(channelGroupsDialog.accept)
        QtCore.QMetaObject.connectSlotsByName(channelGroupsDialog)
        channelGroupsDialog.setTabOrder(self.pushButtonCancel, self.pushButtonApply)
        channelGroupsDialog.setTabOrder(self.pushButtonApply, self.scrollArea)
        channelGroupsDialog.setTabOrder(self.scrollArea, self.radioButtonMEG)
        channelGroupsDialog.setTabOrder(self.radioButtonMEG, self.radioButtonEEG)
        channelGroupsDialog.setTabOrder(self.radioButtonEEG, self.lineEditAdd)
        channelGroupsDialog.setTabOrder(self.lineEditAdd, self.pushButtonAdd)
        channelGroupsDialog.setTabOrder(self.pushButtonAdd, self.listWidgetChannelGroups)
        channelGroupsDialog.setTabOrder(self.listWidgetChannelGroups, self.pushButtonRemove)
        channelGroupsDialog.setTabOrder(self.pushButtonRemove, self.pushButtonReset)
        channelGroupsDialog.setTabOrder(self.pushButtonReset, self.pushButtonSetChannels)

    def retranslateUi(self, channelGroupsDialog):
        _translate = QtCore.QCoreApplication.translate
        channelGroupsDialog.setWindowTitle(_translate("channelGroupsDialog", "Meggie - Channel groups"))
        self.groupBoxChannelGroups.setTitle(_translate("channelGroupsDialog", "Channel groups:"))
        self.pushButtonSetChannels.setText(_translate("channelGroupsDialog", "Set channels"))
        self.pushButtonAdd.setText(_translate("channelGroupsDialog", "Add"))
        self.pushButtonRemove.setText(_translate("channelGroupsDialog", "Remove selected"))
        self.pushButtonReset.setText(_translate("channelGroupsDialog", "Reset to defaults"))
        self.groupBoxChannelType.setTitle(_translate("channelGroupsDialog", "Select channel type:"))
        self.radioButtonMEG.setText(_translate("channelGroupsDialog", "MEG"))
        self.radioButtonEEG.setText(_translate("channelGroupsDialog", "EEG"))
        self.pushButtonCancel.setText(_translate("channelGroupsDialog", "Cancel"))
        self.pushButtonApply.setText(_translate("channelGroupsDialog", "Apply"))




""" Defines TFR class, wraps mne.AverageTFR objects.
"""

import os
import re
import logging

import mne

from meggie.utilities.datatype import Datatype


class TFR(Datatype):
    """ A wrapper for mne.AverageTFR objects.

    Parameters
    ----------
    name : str
        Name of the tfr, used in the UI lists and in the .exp file.
    directory : str
        Absolute path to the data folder, usually workspace/experiment/subject/tfrs.
    params : dict
        Contains additional information about the tfr.
    content : dict of mne.AverageTFR, optional
        Stores mne.AverageTFR objects as values, conditions as keys. If not provided,
        is assumed to be saved to file system earlier.
    """

    def __init__(self, name, directory, params, content=None):
        self._name = name
        self._params = params
        self._directory = directory

        # ensure comments are set to match the keys / conditions
        self._content = content
        if self._content:
            for key in self._content.keys():
                self._content[key].comment = key

    def _get_fname(self, tfr_name):
        """A helper to get fname that work with old versions of meggie too."""
        if tfr_name == '':
            name = self._name + '-tfr.h5'
        else:
            name = self._name + '-' + tfr_name + '-tfr.h5'

        fname = os.path.join(self._directory,
                             name)
        return fname

    def save_content(self):
        """Saves the mne.AverageTFR to h5 files in the tfr directory.
        """
        try:
            for tfr_name, tfr in self._content.items():
                fname = self._get_fname(tfr_name)
                tfr.save(fname, overwrite=True)
        except Exception as exc:
            raise Exception("Writing TFRs failed. Please ensure that the "
                            "entire experiment folder has write permissions.")

    def delete_content(self):
        """Deletes the correct h5 files in the tfr directory"""

        template = self._name + '-' + r'([a-zA-Z1-9_]+)\-tfr\.h5'
        for fname in os.listdir(self._directory):
            match = re.match(template, fname)
            if match:
                try:
                    key = str(match.group(1))
                except Exception as exc:
                    continue

                # if proper condition parameters set,
                # check if the key is in there
                if 'conditions' in self._params:
                    if key not in [str(elem) for elem in
                                   self._params['conditions']]:
                        continue

                os.remove(os.path.join(self._directory, fname))

    def _load_content(self):
        """Handle the loading of the content."""
        self._content = {}
        template = self._name + '-' + r'([a-zA-Z1-9_]+)\-tfr\.h5'
        for fname in os.listdir(self._directory):
            path = None
            if fname == self._name + '-tfr.h5':
                path = os.path.join(self._directory, fname)
                key = ''
            else:
                match = re.match(template, fname)
                if match:
                    try:
                        key = str(match.group(1))
                    except Exception as exc:
                        raise Exception("Unknown file name format.")

                    # if proper condition parameters set,
                    # check if the key is in there
                    if 'conditions' in self._params:
                        if key not in [str(elem) for elem in
                                       self._params['conditions']]:
                            continue

                    path = os.path.join(self._directory,
                                        fname)
            if path:
                self._content[key] = mne.time_frequency.read_tfrs(path)[0]

    @property
    def content(self):
        """Returns the actual mne.AverageTFR objects either
        from cache or from the file system.
        """
        if self._content is None:
            self._load_content()
        return self._content

    @property
    def params(self):
        """Returns additional information stored.
        """
        return self._params

    @property
    def data(self):
        """Returns the actual numerical data of TFRs."""
        data = {}
        for key in self.content.keys():
            data[key] = self.content[key].data
        return data

    @property
    def name(self):
        """Returns the name of the tfr."""
        return self._name

    @property
    def decim(self):
        """Returns the decimation factor used in the creation."""
        return self._params.get('decim')

    @property
    def n_cycles(self):
        """"Returns the number of cycles in used in the creation."""
        return self._params.get('n_cycles')

    @property
    def ch_names(self):
        """Returns the channel names, must read the data first.
        """
        return list(self.content.values())[0].info['ch_names']

    @property
    def times(self):
        """Returns the array of times, must read the data first."""
        return list(self.content.values())[0].times

    @property
    def freqs(self):
        """Returns the array of freqs, must read the data first."""
        return list(self.content.values())[0].freqs

    @property
    def info(self):
        """Returns the info structure, must read the data first."""
        return list(self.content.values())[0].info

    @property
    def evoked_subtracted(self):
        """Returns whether evoked was subtracted in the creation."""
        return self._params.get('evoked_subtracted')



""" Contains implementation for raw montage
"""
import logging

import matplotlib.pyplot as plt
import numpy as np
import mne

from meggie.utilities.messaging import messagebox
from meggie.utilities.messaging import exc_messagebox
from meggie.utilities.threading import threaded

from meggie.actions.raw_montage.dialogs.montageDialogMain import MontageDialog

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action


class Montage(Action):
    """ Shows a dialog for gathering parameters and then
    allows setting montage for EEG.
    """

    @subject_action
    def handler(self, subject, params):
        """
        """
        @threaded
        def montage_fun():
            """
            """
            raw = subject.get_raw()

            head_size = params['head_size']

            if params['custom'] == True:
                montage_fname = params['selection']
                montage = mne.channels.read_custom_montage(
                    montage_fname, head_size=head_size)
            else:
                montage_name = params['selection']
                montage = mne.channels.make_standard_montage(
                    montage_name, head_size=head_size)

            raw.set_montage(montage)
            subject.save()

        montage_fun(do_meanwhile=self.window.update_ui)

    def run(self):
        montage_dialog = MontageDialog(
            self.window, self.experiment, self.handler)
        montage_dialog.show()



"""Contains functions to compare two raws."""

from copy import deepcopy

import mne
import numpy as np


def _prepare_raw_for_changes(raw_from, raw_to):
    """ Creates a new raw with channels interleaved from the two input raws

    """
    raw_to = raw_to.copy()
    raw_from = raw_from.copy()

    raw_to.drop_channels(raw_to.info['bads'])
    raw_from.drop_channels(raw_from.info['bads'])

    ch_names = []
    ch_types = []
    for ch_idx, ch_name in enumerate(raw_from.info['ch_names']):
        ch_type = mne.io.pick.channel_type(raw_from.info, ch_idx)
        ch_names.append(ch_name.strip())
        ch_types.append(ch_type)
        ch_names.append(ch_name.strip() + '*')
        ch_types.append(ch_type)

    new_info = mne.create_info(ch_names, 
                               raw_from.info['sfreq'], 
                               ch_types=ch_types)

    new_info['bads'] = [name for idx, name in enumerate(new_info['ch_names'])
                        if idx % 2 == 0]

    new_data = np.zeros((raw_from._data.shape[0] * 2, raw_from._data.shape[1]))
    new_data[0::2, :] = raw_from._data
    new_data[1::2, :] = raw_to._data

    new_raw = mne.io.RawArray(new_data, 
                              new_info, 
                              first_samp=raw_from.first_samp)

    return new_raw

def compare_raws(raw_from, raw_to):
    """Creates and plots a new raw object with channels from two raws
    interleaved. Can be used to analyse how some action changes
    the raw data.

    Parameters
    ----------
    raw_from : mne.io.Raw
        The original raw.
    raw_to : mne.io.Raw
        The changed raw.

    """
    changes_raw = _prepare_raw_for_changes(raw_from, raw_to)
    changes_raw.plot(color='red', bad_color='blue', title='Comparison plot')



""" Functions for validation.
"""
import re
import numpy as np


def validate_name(name, minlength=1, maxlength=30, fieldname='name'):
    """Validates a name with length and regular expression criteria.
    
    Parameters
    ----------
    name : str or QString
        Name to be validated.
    minlength : int
        The minimum length of the name.
    maxlength : int
        The maximum length of the name.
    fieldname : str
        The name of the field, for good exception messages.

    Returns
    -------
    str
        Name that passed the validation.

    """

    name = str(name)

    if len(name) < minlength:
        raise Exception('You need to set ' + fieldname)

    if len(name) > maxlength:
        raise Exception('Too long ' + fieldname + ' (over ' + str(maxlength) +
                        ' characters.)')

    if not re.match(r'^[A-Za-z0-9_]*$', name):
        raise Exception(fieldname + ' can only contain alphanumeric ' +
                        'characters or underscores')

    return name


def assert_arrays_same(arrays, message='Times do not match'):
    """Checks if arrays in a list are all equal.

    Parameters
    ----------
    arrays : list
        List containing the numpy arrays to be compared.
    message : str
        Message for the exception if arrays are not the same.
    """
    for idx in range(len(arrays)-1):
        try:
            first = np.array(arrays[0]).astype(np.float)
            second = np.array(arrays[idx+1]).astype(np.float)
            np.testing.assert_array_almost_equal(first, second)
        except Exception as exc:
            raise Exception(message)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rereferencingDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_rereferencingDialog(object):
    def setupUi(self, rereferencingDialog):
        rereferencingDialog.setObjectName("rereferencingDialog")
        rereferencingDialog.resize(423, 498)
        self.gridLayout_2 = QtWidgets.QGridLayout(rereferencingDialog)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pushButtonCancel = QtWidgets.QPushButton(rereferencingDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonBatch = QtWidgets.QPushButton(rereferencingDialog)
        self.pushButtonBatch.setObjectName("pushButtonBatch")
        self.horizontalLayout.addWidget(self.pushButtonBatch)
        self.pushButtonApply = QtWidgets.QPushButton(rereferencingDialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayout.addWidget(self.pushButtonApply)
        self.gridLayout_2.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.scrollArea = QtWidgets.QScrollArea(rereferencingDialog)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 403, 447))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBoxRereference = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxRereference.setObjectName("groupBoxRereference")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBoxRereference)
        self.formLayout_2.setObjectName("formLayout_2")
        self.labelSelectChannel = QtWidgets.QLabel(self.groupBoxRereference)
        self.labelSelectChannel.setObjectName("labelSelectChannel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.labelSelectChannel)
        self.comboBoxChannel = QtWidgets.QComboBox(self.groupBoxRereference)
        self.comboBoxChannel.setMinimumSize(QtCore.QSize(120, 0))
        self.comboBoxChannel.setObjectName("comboBoxChannel")
        self.comboBoxChannel.addItem("")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.comboBoxChannel)
        self.gridLayout.addWidget(self.groupBoxRereference, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 2, 0, 1, 1)
        self.groupBoxBatching = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBatching.setObjectName("groupBoxBatching")
        self.gridLayoutBatching = QtWidgets.QGridLayout(self.groupBoxBatching)
        self.gridLayoutBatching.setObjectName("gridLayoutBatching")
        self.batchingWidgetPlaceholder = QtWidgets.QWidget(self.groupBoxBatching)
        self.batchingWidgetPlaceholder.setMinimumSize(QtCore.QSize(300, 300))
        self.batchingWidgetPlaceholder.setObjectName("batchingWidgetPlaceholder")
        self.gridLayoutBatching.addWidget(self.batchingWidgetPlaceholder, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBoxBatching, 1, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout_2.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(rereferencingDialog)
        self.pushButtonCancel.clicked.connect(rereferencingDialog.reject)
        self.pushButtonApply.clicked.connect(rereferencingDialog.accept)
        self.pushButtonBatch.clicked.connect(rereferencingDialog.acceptBatch)
        QtCore.QMetaObject.connectSlotsByName(rereferencingDialog)

    def retranslateUi(self, rereferencingDialog):
        _translate = QtCore.QCoreApplication.translate
        rereferencingDialog.setWindowTitle(_translate("rereferencingDialog", "Meggie - Rereferencing"))
        self.pushButtonCancel.setText(_translate("rereferencingDialog", "Cancel"))
        self.pushButtonBatch.setText(_translate("rereferencingDialog", "Batch"))
        self.pushButtonApply.setText(_translate("rereferencingDialog", "Apply"))
        self.groupBoxRereference.setTitle(_translate("rereferencingDialog", "Rereferencing options"))
        self.labelSelectChannel.setText(_translate("rereferencingDialog", "Select channel:"))
        self.comboBoxChannel.setCurrentText(_translate("rereferencingDialog", "Use average"))
        self.comboBoxChannel.setItemText(0, _translate("rereferencingDialog", "Use average"))
        self.groupBoxBatching.setTitle(_translate("rereferencingDialog", "Batching"))




""" Contains implementation for raw plot projections
"""
import logging

import matplotlib.pyplot as plt
import numpy as np

from meggie.utilities.messaging import messagebox
from meggie.utilities.messaging import exc_messagebox

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action


class PlotProjections(Action):
    """ Shows a plot of existing projection vectors.
    """
    def run(self):

        subject = self.experiment.active_subject
        try:
            self.handler(subject, {})
        except Exception as exc:
            exc_messagebox(self.window, exc)

    @subject_action
    def handler(self, subject, params):
        raw = subject.get_raw()
        if not raw.info['projs']:
            messagebox(self.window, "Data does not contain any projection vectors")
            return
        fig = raw.plot_projs_topomap()

        elems = ['projections', subject.name]
        fig.canvas.set_window_title('_'.join(elems))
        fig.suptitle(' '.join(elems))




""" Contains controlling logic for the evoked implementation.
"""

import logging

import mne
import numpy as np

from meggie.datatypes.evoked.evoked import Evoked

from meggie.utilities.validators import assert_arrays_same

from meggie.utilities.threading import threaded
from meggie.utilities.units import get_unit


@threaded
def group_average_evoked(experiment, evoked_name, groups, new_name):
    """ Computes group average item.
    """
    keys = []
    time_arrays = []
    for group_key, group_subjects in groups.items():
        for subject_name in group_subjects:
            try:
                subject = experiment.subjects.get(subject_name)
                evoked = subject.evoked.get(evoked_name)

                mne_evokeds = evoked.content
                for mne_evoked in mne_evokeds.values():
                    time_arrays.append(mne_evoked.times)
            except Exception as exc:
                continue

    assert_arrays_same(time_arrays)

    grand_evokeds = {}
    for group_key, group_subjects in groups.items():
        for subject in experiment.subjects.values():
            if subject.name not in group_subjects:
                continue

            evoked = subject.evoked.get(evoked_name)
            if not evoked:
                logging.getLogger('ui_logger').warning(
                    evoked_name + ' not present for ' + subject.name)
                continue

            for evoked_item_key, evoked_item in evoked.content.items():
                grand_key = (group_key, evoked_item_key)

                if grand_key not in grand_evokeds:
                    grand_evokeds[grand_key] = []
                grand_evokeds[grand_key].append(evoked_item)

    grand_averages = {}
    new_keys = []
    for key, grand_evoked in grand_evokeds.items():
        new_key = str(key[1]) + '_' + str(key[0])
        if len(grand_evoked) == 1:
            grand_averages[new_key] = grand_evoked[0].copy()
        else:
            grand_averages[new_key] = mne.grand_average(grand_evoked)
        new_keys.append(new_key)

    subject = experiment.active_subject

    evoked_directory = subject.evoked_directory

    params = {'conditions': new_keys,
              'groups': groups}

    grand_average_evoked = Evoked(new_name, evoked_directory, params,
                                  content=grand_averages)

    grand_average_evoked.save_content()
    subject.add(grand_average_evoked, 'evoked')




""" Contains a class for logic of the Subjects.
"""

import os
import logging
import json
import pkg_resources

import mne

import meggie.utilities.filemanager as filemanager

from meggie.mainwindow.dynamic import find_all_datatype_specs


class Subject:
    """ The class for holding subject-specific information
    and subject-specific data.

    Parameters
    ----------
    experiment : meggie.experiment.Experiment
        The experiment to which the subject is created.
    name : str
        Name of the subject.
    raw_fname : str
        Path to the subject data.
    uid : str
        A unique identifier to differentiate between subjects that have 
        same name.
    ica_applied : bool
        Whether ICA has been applied (at least once) to this data.
    rereferenced : bool
        Whether the data has been rereferenced (at least once).
    """

    def __init__(self, experiment, name, raw_fname, uid,
                 ica_applied=False, rereferenced=False):
        self.name = name
        self.raw_fname = raw_fname

        self.uid = uid

        self._raw = None

        self.ica_applied = ica_applied
        self.rereferenced = rereferenced

        self.path = os.path.join(experiment.path,
                                 name)

        datatype_specs = find_all_datatype_specs()
        for source, package, datatype_spec in datatype_specs.values():
            datatype = datatype_spec['id']
            dir_ = datatype_spec['dir']
            setattr(self, datatype, dict())
            setattr(self, datatype + '_directory',
                    os.path.join(self.path, dir_))


    def add(self, dataobject, datatype):
        """ Adds a dataobject of type datatype to the subject.

        Parameters
        ----------
        dataobject : instance of a datatype
            A data object.
        datatype : str
            Name of the datatype.
        """
        container = getattr(self, datatype)
        name = dataobject.name
        container[name] = dataobject

    def remove(self, name, datatype):
        """ Removes a dataobject by name from the subject. 

        Parameters
        ----------
        name : str
            Name of the data object.
        datatype : str
            Name of the datatype.

        """
        container = getattr(self, datatype)
        dataobject = container.pop(name, None)
        try:
            dataobject.delete_content()
        except Exception as exc:
            logging.getLogger('ui_logger').exception('')
            raise IOError('Could not delete ' + str(datatype) +
                          ' from folders')

    @property
    def raw_path(self):
        """ Returns the raw path."""
        path = os.path.join(self.path,
                            self.raw_fname)
        return path

    def get_raw(self, preload=True, verbose='warning'):
        """ Gets the raw object for the subject.

        Reads from the file system if not in the memory already.

        Parameters
        ----------
        preload : bool
            Whether to read the data or only the metadata.
        verbose : str
            Verbose level of read_raw.

        Returns
        -------
        mne.io.Raw
            The raw object.
        """
        if self._raw is not None:
            if preload:
                self._raw.load_data()
            return self._raw
        else:
            try:
                raw = filemanager.open_raw(self.raw_path, preload=preload, 
                                           verbose=verbose)
            except OSError:
                raise IOError("Could not find the raw file.")
            self._raw = raw
            return raw

    def save(self):
        """ Saves the data to the existing path. """
        try:
            filemanager.save_raw(self._raw, self.raw_path)
        except Exception as exc:
            raise Exception("Could not save the raw file. Please ensure "
                            "that the entire experiment folder has "
                            "write permissions.")


    def release_memory(self):
        """ Releases data from the memory.
        """
        if self._raw is not None:
            self._raw = None

    @property
    def has_eeg(self):
        """ Checks if the raw has eeg data present
        """
        raw = self.get_raw(preload=False)
        channels = mne.pick_types(raw.info, eeg=True, meg=False)
        if len(channels) == 0:
            return False
        return True

    @property
    def sss_applied(self):
        """Checks if sss applied.
        """

        try:
            raw = self.get_raw()
            for item in raw.info['proc_history']:
                if 'maxfilter' in item.get('creator', []):
                    return True
        except Exception as exc:
            return False

        return False

    def ensure_folders(self):
        """ When called, checks that the subject folder with all datatype folders
        exist and if not, creates them.
        """
        paths = []
        datatype_specs = find_all_datatype_specs()

        for source, package, datatype_spec in datatype_specs.values():
            datatype = datatype_spec['id']
            path = getattr(self, datatype + '_directory')
            paths.append(path)

        try:
            filemanager.ensure_folders(
                [self.path] + paths)
        except OSError:
            raise OSError("Couldn't create all the necessary folders. "
                          "Please ensure that the experiment folder "
                          "has write permissions everywhere.")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainWindowUi.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1136, 734)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.splitterTopBottom = QtWidgets.QSplitter(self.centralwidget)
        self.splitterTopBottom.setOrientation(QtCore.Qt.Vertical)
        self.splitterTopBottom.setObjectName("splitterTopBottom")
        self.gridLayoutWidget = QtWidgets.QWidget(self.splitterTopBottom)
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.gridLayoutTop = QtWidgets.QGridLayout(self.gridLayoutWidget)
        self.gridLayoutTop.setContentsMargins(0, 0, 0, 0)
        self.gridLayoutTop.setObjectName("gridLayoutTop")
        self.scrollAreaLeft = QtWidgets.QScrollArea(self.gridLayoutWidget)
        self.scrollAreaLeft.setMinimumSize(QtCore.QSize(300, 0))
        self.scrollAreaLeft.setMaximumSize(QtCore.QSize(300, 16777215))
        self.scrollAreaLeft.setFocusPolicy(QtCore.Qt.NoFocus)
        self.scrollAreaLeft.setWidgetResizable(True)
        self.scrollAreaLeft.setObjectName("scrollAreaLeft")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 284, 550))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(250, 550))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBoxSubjects = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxSubjects.setObjectName("groupBoxSubjects")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.groupBoxSubjects)
        self.gridLayout_12.setContentsMargins(9, 9, 9, 9)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.pushButtonAddSubjects = QtWidgets.QPushButton(self.groupBoxSubjects)
        self.pushButtonAddSubjects.setObjectName("pushButtonAddSubjects")
        self.gridLayout_12.addWidget(self.pushButtonAddSubjects, 4, 0, 1, 1)
        self.listWidgetSubjects = QtWidgets.QListWidget(self.groupBoxSubjects)
        self.listWidgetSubjects.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listWidgetSubjects.setObjectName("listWidgetSubjects")
        self.gridLayout_12.addWidget(self.listWidgetSubjects, 1, 0, 1, 1)
        self.pushButtonRemoveSubject = QtWidgets.QPushButton(self.groupBoxSubjects)
        self.pushButtonRemoveSubject.setObjectName("pushButtonRemoveSubject")
        self.gridLayout_12.addWidget(self.pushButtonRemoveSubject, 3, 0, 1, 1)
        self.pushButtonActivateSubject = QtWidgets.QPushButton(self.groupBoxSubjects)
        self.pushButtonActivateSubject.setObjectName("pushButtonActivateSubject")
        self.gridLayout_12.addWidget(self.pushButtonActivateSubject, 2, 0, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxSubjects, 2, 0, 1, 1)
        self.groupBoxExperiment = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxExperiment.setObjectName("groupBoxExperiment")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBoxExperiment)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.labelExperimentAuthorValue = QtWidgets.QLabel(self.groupBoxExperiment)
        self.labelExperimentAuthorValue.setText("")
        self.labelExperimentAuthorValue.setObjectName("labelExperimentAuthorValue")
        self.gridLayout_4.addWidget(self.labelExperimentAuthorValue, 1, 1, 1, 2)
        self.labelExperimentAuthor = QtWidgets.QLabel(self.groupBoxExperiment)
        self.labelExperimentAuthor.setObjectName("labelExperimentAuthor")
        self.gridLayout_4.addWidget(self.labelExperimentAuthor, 1, 0, 1, 1)
        self.labelExperimentNameValue = QtWidgets.QLabel(self.groupBoxExperiment)
        self.labelExperimentNameValue.setText("")
        self.labelExperimentNameValue.setObjectName("labelExperimentNameValue")
        self.gridLayout_4.addWidget(self.labelExperimentNameValue, 0, 1, 1, 2)
        self.labelExperimentName = QtWidgets.QLabel(self.groupBoxExperiment)
        self.labelExperimentName.setObjectName("labelExperimentName")
        self.gridLayout_4.addWidget(self.labelExperimentName, 0, 0, 1, 1)
        self.pushButtonChannelGroups = QtWidgets.QPushButton(self.groupBoxExperiment)
        self.pushButtonChannelGroups.setObjectName("pushButtonChannelGroups")
        self.gridLayout_4.addWidget(self.pushButtonChannelGroups, 2, 0, 1, 3)
        self.pushButtonPipelines = QtWidgets.QPushButton(self.groupBoxExperiment)
        self.pushButtonPipelines.setObjectName("pushButtonPipelines")
        self.gridLayout_4.addWidget(self.pushButtonPipelines, 3, 0, 1, 3)
        self.gridLayout_2.addWidget(self.groupBoxExperiment, 0, 0, 1, 1)
        self.scrollAreaLeft.setWidget(self.scrollAreaWidgetContents)
        self.gridLayoutTop.addWidget(self.scrollAreaLeft, 0, 0, 1, 1)
        self.scrollAreaRight = QtWidgets.QScrollArea(self.gridLayoutWidget)
        self.scrollAreaRight.setFocusPolicy(QtCore.Qt.NoFocus)
        self.scrollAreaRight.setWidgetResizable(True)
        self.scrollAreaRight.setObjectName("scrollAreaRight")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 800, 700))
        self.scrollAreaWidgetContents_2.setMinimumSize(QtCore.QSize(800, 700))
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents_2)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.tabWidget = QtWidgets.QTabWidget(self.scrollAreaWidgetContents_2)
        self.tabWidget.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setMinimumSize(QtCore.QSize(200, 400))
        self.tabWidget.setMaximumSize(QtCore.QSize(20000, 16777215))
        self.tabWidget.setObjectName("tabWidget")
        self.gridLayout_3.addWidget(self.tabWidget, 0, 0, 1, 1)
        self.scrollAreaRight.setWidget(self.scrollAreaWidgetContents_2)
        self.gridLayoutTop.addWidget(self.scrollAreaRight, 0, 1, 1, 1)
        self.textEditConsole = QtWidgets.QTextEdit(self.splitterTopBottom)
        self.textEditConsole.setMinimumSize(QtCore.QSize(0, 0))
        self.textEditConsole.setFocusPolicy(QtCore.Qt.NoFocus)
        self.textEditConsole.setObjectName("textEditConsole")
        self.gridLayout.addWidget(self.splitterTopBottom, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1136, 20))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionCreateExperiment = QtWidgets.QAction(MainWindow)
        self.actionCreateExperiment.setObjectName("actionCreateExperiment")
        self.actionOpenExperiment = QtWidgets.QAction(MainWindow)
        self.actionOpenExperiment.setObjectName("actionOpenExperiment")
        self.actionShowExperimentInfo = QtWidgets.QAction(MainWindow)
        self.actionShowExperimentInfo.setObjectName("actionShowExperimentInfo")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionShowLog = QtWidgets.QAction(MainWindow)
        self.actionShowLog.setObjectName("actionShowLog")
        self.actionPreferences = QtWidgets.QAction(MainWindow)
        self.actionPreferences.setObjectName("actionPreferences")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionActions = QtWidgets.QAction(MainWindow)
        self.actionActions.setObjectName("actionActions")
        self.menuFile.addAction(self.actionCreateExperiment)
        self.menuFile.addAction(self.actionOpenExperiment)
        self.menuFile.addAction(self.actionQuit)
        self.menuTools.addAction(self.actionActions)
        self.menuTools.addAction(self.actionPreferences)
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.scrollAreaLeft, self.pushButtonChannelGroups)
        MainWindow.setTabOrder(self.pushButtonChannelGroups, self.listWidgetSubjects)
        MainWindow.setTabOrder(self.listWidgetSubjects, self.pushButtonActivateSubject)
        MainWindow.setTabOrder(self.pushButtonActivateSubject, self.pushButtonRemoveSubject)
        MainWindow.setTabOrder(self.pushButtonRemoveSubject, self.pushButtonAddSubjects)
        MainWindow.setTabOrder(self.pushButtonAddSubjects, self.scrollAreaRight)
        MainWindow.setTabOrder(self.scrollAreaRight, self.tabWidget)
        MainWindow.setTabOrder(self.tabWidget, self.textEditConsole)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Meggie"))
        self.groupBoxSubjects.setTitle(_translate("MainWindow", "Subjects"))
        self.pushButtonAddSubjects.setText(_translate("MainWindow", "Add new..."))
        self.pushButtonRemoveSubject.setText(_translate("MainWindow", "Remove selected"))
        self.pushButtonActivateSubject.setText(_translate("MainWindow", "Activate selected"))
        self.groupBoxExperiment.setTitle(_translate("MainWindow", "Experiment"))
        self.labelExperimentAuthor.setText(_translate("MainWindow", "Author:"))
        self.labelExperimentName.setText(_translate("MainWindow", "Name:"))
        self.pushButtonChannelGroups.setText(_translate("MainWindow", "Channel groups..."))
        self.pushButtonPipelines.setText(_translate("MainWindow", "Change pipeline..."))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuTools.setTitle(_translate("MainWindow", "Tools"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.actionCreateExperiment.setText(_translate("MainWindow", "Create new experiment..."))
        self.actionOpenExperiment.setText(_translate("MainWindow", "Open experiment..."))
        self.actionShowExperimentInfo.setText(_translate("MainWindow", "Show experiment info..."))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
        self.actionShowLog.setText(_translate("MainWindow", "Show mne log..."))
        self.actionPreferences.setText(_translate("MainWindow", "Preferences..."))
        self.actionAbout.setText(_translate("MainWindow", "About..."))
        self.actionActions.setText(_translate("MainWindow", "Show past actions..."))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pipelineDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_pipelineDialog(object):
    def setupUi(self, pipelineDialog):
        pipelineDialog.setObjectName("pipelineDialog")
        pipelineDialog.resize(406, 195)
        self.gridLayout = QtWidgets.QGridLayout(pipelineDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(pipelineDialog)
        self.scrollArea.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 386, 144))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 1, 0, 1, 1)
        self.groupBoxPipeline = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxPipeline.setObjectName("groupBoxPipeline")
        self.gridLayoutPipeline = QtWidgets.QGridLayout(self.groupBoxPipeline)
        self.gridLayoutPipeline.setObjectName("gridLayoutPipeline")
        self.gridLayout_2.addWidget(self.groupBoxPipeline, 0, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(pipelineDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonApply = QtWidgets.QPushButton(pipelineDialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayout.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(pipelineDialog)
        self.pushButtonCancel.clicked.connect(pipelineDialog.reject)
        self.pushButtonApply.clicked.connect(pipelineDialog.accept)
        QtCore.QMetaObject.connectSlotsByName(pipelineDialog)

    def retranslateUi(self, pipelineDialog):
        _translate = QtCore.QCoreApplication.translate
        pipelineDialog.setWindowTitle(_translate("pipelineDialog", "Meggie - Select pipeline"))
        self.groupBoxPipeline.setTitle(_translate("pipelineDialog", "Select pipeline:"))
        self.pushButtonCancel.setText(_translate("pipelineDialog", "Cancel"))
        self.pushButtonApply.setText(_translate("pipelineDialog", "Apply"))



""" Contains a class for logic of the short question box.
"""

from PyQt5 import QtWidgets

from meggie.utilities.dialogs.shortQuestionBoxUi import Ui_shortQuestionBox


class shortQuestionBox(QtWidgets.QDialog):
    """ Contains logic for the short question box.
    """

    def __init__(self, message, parent, handler):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_shortQuestionBox()
        self.ui.setupUi(self)

        self.setWindowTitle('Meggie - Question')
        self.ui.labelMessage.setText(message)

        self.handler = handler

    def accept(self):
        self.handler(True)
        super(shortQuestionBox, self).accept()

    def reject(self):
        self.handler(False)
        super(shortQuestionBox, self).reject()



""" Defines Epochs class, wraps mne.Epochs objects.
"""
import os
import logging

import mne

from meggie.utilities.datatype import Datatype


class Epochs(Datatype):
    """ A wrapper for mne.Epochs objects.

    Parameters
    ----------
    name : str
        Name of the epochs, used in the UI lists and in the .exp file.
    directory : str
        Absolute path to the data folder, usually workspace/experiment/subject/epochs.
    params : dict
        Contains additional information about the epochs, 
        such as the events used in the construction
    content : instance of mne.Epochs, optional
        A mne.Epochs object. If not provided, is assumed to be
        saved to file system earlier.
    """

    def __init__(self, name, directory, params, content=None):
        self._name = name
        self._content = content
        self._params = params
        self._path = os.path.join(directory, name + '.fif')

    @property
    def content(self):
        """Returns the actual mne.Epochs, either from cache or
        from the file system.
        """
        if self._content is not None:
            return self._content
        else:
            try:
                self._content = mne.read_epochs(self._path)
                return self._content
            except IOError:
                raise Exception('Reading epochs failed.')

    @property
    def name(self):
        """Returns the name of the collection.
        """
        return self._name

    @property
    def count(self):
        """Return the number of epochs in the collection.
        """
        return len(self.content.events)

    @property
    def params(self):
        """Returns additional information stored.
        """
        return self._params

    def delete_content(self):
        """Deletes the fif file from the files system
        """
        os.remove(self._path)

    def save_content(self):
        """Saves the mne.Epochs to a fif file in the epochs
        directory """
        try:
            self._content.save(self._path, overwrite=True)
        except Exception as exc:
            raise Exception("Writing epochs failed. Please ensure that "
                            "the entire experiment folder has write permissions")




""" Contains implementation for delete tfr from all
"""
import logging

import numpy as np

from meggie.utilities.messaging import exc_messagebox

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action


class DeleteTFRFromAll(Action):
    """ Deletes TFR of selected name from all subjects
    """

    def run(self):

        try:
            selected_name = self.data['outputs']['tfr'][0]
        except IndexError as exc:
            return

        for subject in self.experiment.subjects.values():
            if selected_name in subject.tfr:
                try:
                    self.handler(subject, {'name': selected_name})
                except Exception as exc:
                    logging.getLogger('ui_logger').exception('')
                    logging.getLogger('ui_logger').warning(
                        'Could not remove TFR for ' +
                        subject.name)

        try:
            self.experiment.save_experiment_settings()
        except Exception as exc:
            exc_messagebox(self.window, exc)

        self.window.initialize_ui()

    @subject_action
    def handler(self, subject, params):
        subject.remove(params['name'], 'tfr')





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'filterDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DialogFilter(object):
    def setupUi(self, DialogFilter):
        DialogFilter.setObjectName("DialogFilter")
        DialogFilter.resize(712, 697)
        self.gridLayout = QtWidgets.QGridLayout(DialogFilter)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pushButtonCancel = QtWidgets.QPushButton(DialogFilter)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonBatch = QtWidgets.QPushButton(DialogFilter)
        self.pushButtonBatch.setObjectName("pushButtonBatch")
        self.horizontalLayout.addWidget(self.pushButtonBatch)
        self.pushButtonApply = QtWidgets.QPushButton(DialogFilter)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayout.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.horizontalLayout, 2, 0, 1, 1)
        self.scrollArea = QtWidgets.QScrollArea(DialogFilter)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 680, 785))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.groupBoxBandstop = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBandstop.setObjectName("groupBoxBandstop")
        self.formLayout = QtWidgets.QFormLayout(self.groupBoxBandstop)
        self.formLayout.setObjectName("formLayout")
        self.checkBoxBandstop = QtWidgets.QCheckBox(self.groupBoxBandstop)
        self.checkBoxBandstop.setChecked(False)
        self.checkBoxBandstop.setObjectName("checkBoxBandstop")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.checkBoxBandstop)
        self.doubleSpinBoxBandstopFreq = QtWidgets.QDoubleSpinBox(self.groupBoxBandstop)
        self.doubleSpinBoxBandstopFreq.setEnabled(False)
        self.doubleSpinBoxBandstopFreq.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxBandstopFreq.setDecimals(3)
        self.doubleSpinBoxBandstopFreq.setMaximum(10000.0)
        self.doubleSpinBoxBandstopFreq.setSingleStep(1.0)
        self.doubleSpinBoxBandstopFreq.setProperty("value", 50.0)
        self.doubleSpinBoxBandstopFreq.setObjectName("doubleSpinBoxBandstopFreq")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxBandstopFreq)
        self.checkBoxBandstop2 = QtWidgets.QCheckBox(self.groupBoxBandstop)
        self.checkBoxBandstop2.setObjectName("checkBoxBandstop2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.checkBoxBandstop2)
        self.doubleSpinBoxBandstopFreq2 = QtWidgets.QDoubleSpinBox(self.groupBoxBandstop)
        self.doubleSpinBoxBandstopFreq2.setEnabled(False)
        self.doubleSpinBoxBandstopFreq2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxBandstopFreq2.setDecimals(3)
        self.doubleSpinBoxBandstopFreq2.setMaximum(10000.0)
        self.doubleSpinBoxBandstopFreq2.setSingleStep(1.0)
        self.doubleSpinBoxBandstopFreq2.setProperty("value", 100.0)
        self.doubleSpinBoxBandstopFreq2.setObjectName("doubleSpinBoxBandstopFreq2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxBandstopFreq2)
        self.labelBandStopNotchWidth = QtWidgets.QLabel(self.groupBoxBandstop)
        self.labelBandStopNotchWidth.setObjectName("labelBandStopNotchWidth")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.labelBandStopNotchWidth)
        self.doubleSpinBoxBandstopWidth = QtWidgets.QDoubleSpinBox(self.groupBoxBandstop)
        self.doubleSpinBoxBandstopWidth.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxBandstopWidth.setDecimals(3)
        self.doubleSpinBoxBandstopWidth.setMaximum(10000.0)
        self.doubleSpinBoxBandstopWidth.setSingleStep(1.0)
        self.doubleSpinBoxBandstopWidth.setProperty("value", 1.0)
        self.doubleSpinBoxBandstopWidth.setObjectName("doubleSpinBoxBandstopWidth")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxBandstopWidth)
        self.labelTransBw = QtWidgets.QLabel(self.groupBoxBandstop)
        self.labelTransBw.setObjectName("labelTransBw")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.labelTransBw)
        self.doubleSpinBoxNotchTransBw = QtWidgets.QDoubleSpinBox(self.groupBoxBandstop)
        self.doubleSpinBoxNotchTransBw.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxNotchTransBw.setDecimals(3)
        self.doubleSpinBoxNotchTransBw.setMinimum(0.1)
        self.doubleSpinBoxNotchTransBw.setSingleStep(0.1)
        self.doubleSpinBoxNotchTransBw.setProperty("value", 0.5)
        self.doubleSpinBoxNotchTransBw.setObjectName("doubleSpinBoxNotchTransBw")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxNotchTransBw)
        self.labelBandstopFilterLength = QtWidgets.QLabel(self.groupBoxBandstop)
        self.labelBandstopFilterLength.setObjectName("labelBandstopFilterLength")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.labelBandstopFilterLength)
        self.doubleSpinBoxBandStopLength = QtWidgets.QDoubleSpinBox(self.groupBoxBandstop)
        self.doubleSpinBoxBandStopLength.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxBandStopLength.setSingleStep(0.1)
        self.doubleSpinBoxBandStopLength.setProperty("value", 10.0)
        self.doubleSpinBoxBandStopLength.setObjectName("doubleSpinBoxBandStopLength")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxBandStopLength)
        self.gridLayout_6.addWidget(self.groupBoxBandstop, 1, 0, 1, 1)
        self.groupBoxBandpass = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBandpass.setObjectName("groupBoxBandpass")
        self.formLayout_5 = QtWidgets.QFormLayout(self.groupBoxBandpass)
        self.formLayout_5.setObjectName("formLayout_5")
        self.checkBoxHighpass = QtWidgets.QCheckBox(self.groupBoxBandpass)
        self.checkBoxHighpass.setChecked(True)
        self.checkBoxHighpass.setObjectName("checkBoxHighpass")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.checkBoxHighpass)
        self.doubleSpinBoxHighpassCutoff = QtWidgets.QDoubleSpinBox(self.groupBoxBandpass)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.doubleSpinBoxHighpassCutoff.sizePolicy().hasHeightForWidth())
        self.doubleSpinBoxHighpassCutoff.setSizePolicy(sizePolicy)
        self.doubleSpinBoxHighpassCutoff.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxHighpassCutoff.setDecimals(3)
        self.doubleSpinBoxHighpassCutoff.setMaximum(1000000000.0)
        self.doubleSpinBoxHighpassCutoff.setSingleStep(1.0)
        self.doubleSpinBoxHighpassCutoff.setProperty("value", 1.0)
        self.doubleSpinBoxHighpassCutoff.setObjectName("doubleSpinBoxHighpassCutoff")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxHighpassCutoff)
        self.checkBoxLowpass = QtWidgets.QCheckBox(self.groupBoxBandpass)
        self.checkBoxLowpass.setChecked(True)
        self.checkBoxLowpass.setObjectName("checkBoxLowpass")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.checkBoxLowpass)
        self.doubleSpinBoxLowpassCutoff = QtWidgets.QDoubleSpinBox(self.groupBoxBandpass)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.doubleSpinBoxLowpassCutoff.sizePolicy().hasHeightForWidth())
        self.doubleSpinBoxLowpassCutoff.setSizePolicy(sizePolicy)
        self.doubleSpinBoxLowpassCutoff.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxLowpassCutoff.setDecimals(3)
        self.doubleSpinBoxLowpassCutoff.setMaximum(10000.0)
        self.doubleSpinBoxLowpassCutoff.setSingleStep(1.0)
        self.doubleSpinBoxLowpassCutoff.setProperty("value", 40.0)
        self.doubleSpinBoxLowpassCutoff.setObjectName("doubleSpinBoxLowpassCutoff")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxLowpassCutoff)
        self.labelTransitionWidth = QtWidgets.QLabel(self.groupBoxBandpass)
        self.labelTransitionWidth.setObjectName("labelTransitionWidth")
        self.formLayout_5.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.labelTransitionWidth)
        self.doubleSpinBoxTransBandwidth = QtWidgets.QDoubleSpinBox(self.groupBoxBandpass)
        self.doubleSpinBoxTransBandwidth.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxTransBandwidth.setDecimals(3)
        self.doubleSpinBoxTransBandwidth.setMaximum(10000.0)
        self.doubleSpinBoxTransBandwidth.setSingleStep(0.1)
        self.doubleSpinBoxTransBandwidth.setProperty("value", 0.5)
        self.doubleSpinBoxTransBandwidth.setObjectName("doubleSpinBoxTransBandwidth")
        self.formLayout_5.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxTransBandwidth)
        self.labelFilterLength = QtWidgets.QLabel(self.groupBoxBandpass)
        self.labelFilterLength.setObjectName("labelFilterLength")
        self.formLayout_5.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.labelFilterLength)
        self.doubleSpinBoxLength = QtWidgets.QDoubleSpinBox(self.groupBoxBandpass)
        self.doubleSpinBoxLength.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBoxLength.setSingleStep(0.1)
        self.doubleSpinBoxLength.setProperty("value", 10.0)
        self.doubleSpinBoxLength.setObjectName("doubleSpinBoxLength")
        self.formLayout_5.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxLength)
        self.gridLayout_6.addWidget(self.groupBoxBandpass, 0, 0, 1, 1)
        self.groupBoxPreview = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxPreview.setObjectName("groupBoxPreview")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBoxPreview)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.pushButtonPreview = QtWidgets.QPushButton(self.groupBoxPreview)
        self.pushButtonPreview.setEnabled(True)
        self.pushButtonPreview.setObjectName("pushButtonPreview")
        self.gridLayout_3.addWidget(self.pushButtonPreview, 0, 0, 1, 1)
        self.gridLayout_6.addWidget(self.groupBoxPreview, 2, 0, 1, 1)
        self.groupBoxBatching = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBatching.setObjectName("groupBoxBatching")
        self.gridLayoutBatching = QtWidgets.QGridLayout(self.groupBoxBatching)
        self.gridLayoutBatching.setObjectName("gridLayoutBatching")
        self.batchingWidgetPlaceholder = QtWidgets.QWidget(self.groupBoxBatching)
        self.batchingWidgetPlaceholder.setMinimumSize(QtCore.QSize(300, 300))
        self.batchingWidgetPlaceholder.setObjectName("batchingWidgetPlaceholder")
        self.gridLayoutBatching.addWidget(self.batchingWidgetPlaceholder, 0, 0, 1, 1)
        self.gridLayout_6.addWidget(self.groupBoxBatching, 3, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(DialogFilter)
        self.pushButtonApply.clicked.connect(DialogFilter.accept)
        self.pushButtonCancel.clicked.connect(DialogFilter.reject)
        self.pushButtonBatch.clicked.connect(DialogFilter.acceptBatch)
        self.checkBoxBandstop.toggled['bool'].connect(self.doubleSpinBoxBandstopFreq.setEnabled)
        self.checkBoxBandstop2.toggled['bool'].connect(self.doubleSpinBoxBandstopFreq2.setEnabled)
        self.checkBoxHighpass.toggled['bool'].connect(self.doubleSpinBoxHighpassCutoff.setEnabled)
        self.checkBoxLowpass.toggled['bool'].connect(self.doubleSpinBoxLowpassCutoff.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(DialogFilter)

    def retranslateUi(self, DialogFilter):
        _translate = QtCore.QCoreApplication.translate
        DialogFilter.setWindowTitle(_translate("DialogFilter", "Meggie - Filtering"))
        self.pushButtonCancel.setText(_translate("DialogFilter", "Cancel"))
        self.pushButtonBatch.setText(_translate("DialogFilter", "Batch"))
        self.pushButtonApply.setText(_translate("DialogFilter", "Apply"))
        self.groupBoxBandstop.setTitle(_translate("DialogFilter", "Bandstop filter"))
        self.checkBoxBandstop.setText(_translate("DialogFilter", "BandStop filter (notch)"))
        self.doubleSpinBoxBandstopFreq.setSuffix(_translate("DialogFilter", " Hz"))
        self.checkBoxBandstop2.setText(_translate("DialogFilter", "BandStop filter (notch)"))
        self.doubleSpinBoxBandstopFreq2.setSuffix(_translate("DialogFilter", " Hz"))
        self.labelBandStopNotchWidth.setText(_translate("DialogFilter", "Notch band width:"))
        self.doubleSpinBoxBandstopWidth.setSuffix(_translate("DialogFilter", " Hz"))
        self.labelTransBw.setText(_translate("DialogFilter", "Transition band width:"))
        self.doubleSpinBoxNotchTransBw.setSuffix(_translate("DialogFilter", " Hz"))
        self.labelBandstopFilterLength.setText(_translate("DialogFilter", "Filter length:"))
        self.doubleSpinBoxBandStopLength.setSuffix(_translate("DialogFilter", "s"))
        self.groupBoxBandpass.setTitle(_translate("DialogFilter", "Bandpass filter"))
        self.checkBoxHighpass.setText(_translate("DialogFilter", "Highpass filter"))
        self.doubleSpinBoxHighpassCutoff.setSuffix(_translate("DialogFilter", " Hz"))
        self.checkBoxLowpass.setText(_translate("DialogFilter", "Lowpass filter"))
        self.doubleSpinBoxLowpassCutoff.setSuffix(_translate("DialogFilter", " Hz"))
        self.labelTransitionWidth.setText(_translate("DialogFilter", "Transition band width:"))
        self.doubleSpinBoxTransBandwidth.setSuffix(_translate("DialogFilter", " Hz"))
        self.labelFilterLength.setText(_translate("DialogFilter", "Filter length:"))
        self.doubleSpinBoxLength.setSuffix(_translate("DialogFilter", "s"))
        self.groupBoxPreview.setTitle(_translate("DialogFilter", "Preview"))
        self.pushButtonPreview.setText(_translate("DialogFilter", "Preview with given filters"))
        self.groupBoxBatching.setTitle(_translate("DialogFilter", "Batching"))




""" Contains implementation for raw plot
"""
import logging

import matplotlib.pyplot as plt
import numpy as np

from meggie.utilities.events import find_stim_channel
from meggie.utilities.events import find_events
from meggie.utilities.messaging import exc_messagebox

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action


class PlotRaw(Action):
    """ Shows a mne raw plot and saves the changes to FS
    if annotations or bads have changed.
    """

    def run(self):

        subject = self.experiment.active_subject
        raw = subject.get_raw()
        if not raw:
            return

        old_bads = raw.info['bads'].copy()
        old_annotations = raw.annotations.copy()

        # find events
        stim_ch = find_stim_channel(raw)
        if not stim_ch:
            events = None
        else:
            events = find_events(raw, stim_ch=stim_ch)

        def handle_close(event):
            bads_changed = (sorted(raw.info['bads']) != sorted(old_bads))

            annotations_changed = False
            if len(raw.annotations) != len(old_annotations):
                annotations_changed = True
            elif not np.allclose(raw.annotations.onset, old_annotations.onset):
                annotations_changed = True

            params = {}
            params['bads'] = raw.info['bads']
            params['annotations'] = list(raw.annotations)
            params['bads_changed'] = bads_changed
            params['annotations_changed'] = annotations_changed

            try:
                self.handler(subject, params)
            except Exception as exc:
                exc_messagebox(self.window, exc)

        fig = raw.plot(events=events, show=False)
        fig.canvas.mpl_connect('close_event', handle_close)
        plt.show()

    @subject_action
    def handler(self, subject, params):

        if params['bads_changed']:
            logging.getLogger('ui_logger').info('Bads changed!')
        if params['annotations_changed']:
            logging.getLogger('ui_logger').info('Annotations changed!')

        if params['bads_changed'] or params['annotations_changed']:
            subject.save()

        self.window.initialize_ui()




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'designer_ui_files/createEpochsFromEventsDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CreateEpochsFromEventsDialog(object):
    def setupUi(self, CreateEpochsFromEventsDialog):
        CreateEpochsFromEventsDialog.setObjectName("CreateEpochsFromEventsDialog")
        CreateEpochsFromEventsDialog.setWindowModality(QtCore.Qt.NonModal)
        CreateEpochsFromEventsDialog.resize(777, 783)
        self.gridLayout = QtWidgets.QGridLayout(CreateEpochsFromEventsDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setContentsMargins(0, 0, -1, -1)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem)
        self.pushButtonCancel = QtWidgets.QPushButton(CreateEpochsFromEventsDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout_10.addWidget(self.pushButtonCancel)
        self.pushButtonBatch = QtWidgets.QPushButton(CreateEpochsFromEventsDialog)
        self.pushButtonBatch.setObjectName("pushButtonBatch")
        self.horizontalLayout_10.addWidget(self.pushButtonBatch)
        self.pushButtonApply = QtWidgets.QPushButton(CreateEpochsFromEventsDialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayout_10.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.horizontalLayout_10, 2, 0, 1, 1)
        self.scrollArea = QtWidgets.QScrollArea(CreateEpochsFromEventsDialog)
        self.scrollArea.setEnabled(True)
        self.scrollArea.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setFrameShadow(QtWidgets.QFrame.Plain)
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 745, 747))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.groupBoxEventsList = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxEventsList.setObjectName("groupBoxEventsList")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBoxEventsList)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.listWidgetEvents = QtWidgets.QListWidget(self.groupBoxEventsList)
        self.listWidgetEvents.setObjectName("listWidgetEvents")
        self.gridLayout_3.addWidget(self.listWidgetEvents, 0, 0, 1, 1)
        self.gridLayout_7.addWidget(self.groupBoxEventsList, 1, 1, 2, 1)
        self.groupBoxRejection = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxRejection.setObjectName("groupBoxRejection")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBoxRejection)
        self.formLayout_2.setObjectName("formLayout_2")
        self.checkBoxGrad = QtWidgets.QCheckBox(self.groupBoxRejection)
        self.checkBoxGrad.setChecked(True)
        self.checkBoxGrad.setObjectName("checkBoxGrad")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.checkBoxGrad)
        self.doubleSpinBoxGradReject = QtWidgets.QDoubleSpinBox(self.groupBoxRejection)
        self.doubleSpinBoxGradReject.setPrefix("")
        self.doubleSpinBoxGradReject.setMinimum(-1.0)
        self.doubleSpinBoxGradReject.setMaximum(1000000000.0)
        self.doubleSpinBoxGradReject.setSingleStep(100.0)
        self.doubleSpinBoxGradReject.setProperty("value", 3000.0)
        self.doubleSpinBoxGradReject.setObjectName("doubleSpinBoxGradReject")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxGradReject)
        self.checkBoxMag = QtWidgets.QCheckBox(self.groupBoxRejection)
        self.checkBoxMag.setChecked(True)
        self.checkBoxMag.setObjectName("checkBoxMag")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.checkBoxMag)
        self.doubleSpinBoxMagReject = QtWidgets.QDoubleSpinBox(self.groupBoxRejection)
        self.doubleSpinBoxMagReject.setMinimum(-1.0)
        self.doubleSpinBoxMagReject.setMaximum(1000000000.0)
        self.doubleSpinBoxMagReject.setSingleStep(100.0)
        self.doubleSpinBoxMagReject.setProperty("value", 4000.0)
        self.doubleSpinBoxMagReject.setObjectName("doubleSpinBoxMagReject")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxMagReject)
        self.checkBoxEeg = QtWidgets.QCheckBox(self.groupBoxRejection)
        self.checkBoxEeg.setChecked(True)
        self.checkBoxEeg.setObjectName("checkBoxEeg")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.checkBoxEeg)
        self.doubleSpinBoxEEGReject = QtWidgets.QDoubleSpinBox(self.groupBoxRejection)
        self.doubleSpinBoxEEGReject.setEnabled(True)
        self.doubleSpinBoxEEGReject.setMinimum(-1.0)
        self.doubleSpinBoxEEGReject.setMaximum(1000000000.0)
        self.doubleSpinBoxEEGReject.setProperty("value", 70.0)
        self.doubleSpinBoxEEGReject.setObjectName("doubleSpinBoxEEGReject")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxEEGReject)
        self.gridLayout_7.addWidget(self.groupBoxRejection, 0, 1, 1, 1)
        self.groupBoxEpochs = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxEpochs.setObjectName("groupBoxEpochs")
        self.formLayout = QtWidgets.QFormLayout(self.groupBoxEpochs)
        self.formLayout.setObjectName("formLayout")
        self.labelCollectionName = QtWidgets.QLabel(self.groupBoxEpochs)
        self.labelCollectionName.setObjectName("labelCollectionName")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.labelCollectionName)
        self.lineEditCollectionName = QtWidgets.QLineEdit(self.groupBoxEpochs)
        self.lineEditCollectionName.setObjectName("lineEditCollectionName")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineEditCollectionName)
        self.labelTmin = QtWidgets.QLabel(self.groupBoxEpochs)
        self.labelTmin.setObjectName("labelTmin")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.labelTmin)
        self.doubleSpinBoxTmin = QtWidgets.QDoubleSpinBox(self.groupBoxEpochs)
        self.doubleSpinBoxTmin.setDecimals(4)
        self.doubleSpinBoxTmin.setMinimum(-100.0)
        self.doubleSpinBoxTmin.setMaximum(100.99)
        self.doubleSpinBoxTmin.setSingleStep(0.1)
        self.doubleSpinBoxTmin.setProperty("value", -0.2)
        self.doubleSpinBoxTmin.setObjectName("doubleSpinBoxTmin")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxTmin)
        self.labelTmax = QtWidgets.QLabel(self.groupBoxEpochs)
        self.labelTmax.setObjectName("labelTmax")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.labelTmax)
        self.doubleSpinBoxTmax = QtWidgets.QDoubleSpinBox(self.groupBoxEpochs)
        self.doubleSpinBoxTmax.setDecimals(4)
        self.doubleSpinBoxTmax.setMinimum(-100.0)
        self.doubleSpinBoxTmax.setMaximum(100.9)
        self.doubleSpinBoxTmax.setSingleStep(0.1)
        self.doubleSpinBoxTmax.setProperty("value", 0.5)
        self.doubleSpinBoxTmax.setObjectName("doubleSpinBoxTmax")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxTmax)
        self.labelBaselineStart = QtWidgets.QLabel(self.groupBoxEpochs)
        self.labelBaselineStart.setObjectName("labelBaselineStart")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.labelBaselineStart)
        self.doubleSpinBoxBaselineStart = QtWidgets.QDoubleSpinBox(self.groupBoxEpochs)
        self.doubleSpinBoxBaselineStart.setDecimals(4)
        self.doubleSpinBoxBaselineStart.setMinimum(-100.0)
        self.doubleSpinBoxBaselineStart.setMaximum(100.99)
        self.doubleSpinBoxBaselineStart.setSingleStep(0.1)
        self.doubleSpinBoxBaselineStart.setProperty("value", -0.2)
        self.doubleSpinBoxBaselineStart.setObjectName("doubleSpinBoxBaselineStart")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxBaselineStart)
        self.labelBaselineEnd = QtWidgets.QLabel(self.groupBoxEpochs)
        self.labelBaselineEnd.setObjectName("labelBaselineEnd")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.labelBaselineEnd)
        self.doubleSpinBoxBaselineEnd = QtWidgets.QDoubleSpinBox(self.groupBoxEpochs)
        self.doubleSpinBoxBaselineEnd.setDecimals(4)
        self.doubleSpinBoxBaselineEnd.setMinimum(-100.0)
        self.doubleSpinBoxBaselineEnd.setMaximum(100.99)
        self.doubleSpinBoxBaselineEnd.setSingleStep(0.1)
        self.doubleSpinBoxBaselineEnd.setProperty("value", 0.0)
        self.doubleSpinBoxBaselineEnd.setObjectName("doubleSpinBoxBaselineEnd")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxBaselineEnd)
        self.labelDelay = QtWidgets.QLabel(self.groupBoxEpochs)
        self.labelDelay.setObjectName("labelDelay")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.labelDelay)
        self.doubleSpinBoxDelay = QtWidgets.QDoubleSpinBox(self.groupBoxEpochs)
        self.doubleSpinBoxDelay.setDecimals(4)
        self.doubleSpinBoxDelay.setMinimum(-100.0)
        self.doubleSpinBoxDelay.setMaximum(100.0)
        self.doubleSpinBoxDelay.setSingleStep(0.1)
        self.doubleSpinBoxDelay.setObjectName("doubleSpinBoxDelay")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBoxDelay)
        self.gridLayout_7.addWidget(self.groupBoxEpochs, 0, 0, 1, 1)
        self.groupBoxBatching = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBatching.setObjectName("groupBoxBatching")
        self.gridLayoutBatching = QtWidgets.QGridLayout(self.groupBoxBatching)
        self.gridLayoutBatching.setObjectName("gridLayoutBatching")
        self.batchingWidgetPlaceholder = QtWidgets.QWidget(self.groupBoxBatching)
        self.batchingWidgetPlaceholder.setMinimumSize(QtCore.QSize(300, 300))
        self.batchingWidgetPlaceholder.setObjectName("batchingWidgetPlaceholder")
        self.gridLayoutBatching.addWidget(self.batchingWidgetPlaceholder, 0, 0, 1, 1)
        self.gridLayout_7.addWidget(self.groupBoxBatching, 3, 0, 1, 2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem1, 4, 0, 1, 1)
        self.groupBoxEvent = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxEvent.setObjectName("groupBoxEvent")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBoxEvent)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.labelMask = QtWidgets.QLabel(self.groupBoxEvent)
        self.labelMask.setObjectName("labelMask")
        self.gridLayout_2.addWidget(self.labelMask, 1, 0, 1, 1)
        self.labelEventID = QtWidgets.QLabel(self.groupBoxEvent)
        self.labelEventID.setObjectName("labelEventID")
        self.gridLayout_2.addWidget(self.labelEventID, 0, 0, 1, 1)
        self.spinBoxMask = QtWidgets.QSpinBox(self.groupBoxEvent)
        self.spinBoxMask.setMaximum(9999999)
        self.spinBoxMask.setObjectName("spinBoxMask")
        self.gridLayout_2.addWidget(self.spinBoxMask, 1, 1, 1, 1)
        self.spinBoxEventID = QtWidgets.QSpinBox(self.groupBoxEvent)
        self.spinBoxEventID.setMinimum(0)
        self.spinBoxEventID.setMaximum(9999999)
        self.spinBoxEventID.setObjectName("spinBoxEventID")
        self.gridLayout_2.addWidget(self.spinBoxEventID, 0, 1, 1, 1)
        self.pushButtonEdit = QtWidgets.QPushButton(self.groupBoxEvent)
        self.pushButtonEdit.setObjectName("pushButtonEdit")
        self.gridLayout_2.addWidget(self.pushButtonEdit, 0, 2, 1, 1)
        self.pushButtonHelp = QtWidgets.QPushButton(self.groupBoxEvent)
        self.pushButtonHelp.setObjectName("pushButtonHelp")
        self.gridLayout_2.addWidget(self.pushButtonHelp, 1, 2, 1, 1)
        self.pushButtonAdd = QtWidgets.QPushButton(self.groupBoxEvent)
        self.pushButtonAdd.setObjectName("pushButtonAdd")
        self.gridLayout_2.addWidget(self.pushButtonAdd, 3, 0, 1, 3)
        self.pushButtonClear = QtWidgets.QPushButton(self.groupBoxEvent)
        self.pushButtonClear.setObjectName("pushButtonClear")
        self.gridLayout_2.addWidget(self.pushButtonClear, 4, 0, 1, 3)
        self.gridLayout_7.addWidget(self.groupBoxEvent, 1, 0, 2, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(CreateEpochsFromEventsDialog)
        self.pushButtonApply.clicked.connect(CreateEpochsFromEventsDialog.accept)
        self.pushButtonCancel.clicked.connect(CreateEpochsFromEventsDialog.reject)
        self.checkBoxGrad.toggled['bool'].connect(self.doubleSpinBoxGradReject.setEnabled)
        self.checkBoxMag.toggled['bool'].connect(self.doubleSpinBoxMagReject.setEnabled)
        self.checkBoxEeg.toggled['bool'].connect(self.doubleSpinBoxEEGReject.setEnabled)
        self.pushButtonBatch.clicked.connect(CreateEpochsFromEventsDialog.acceptBatch)
        QtCore.QMetaObject.connectSlotsByName(CreateEpochsFromEventsDialog)
        CreateEpochsFromEventsDialog.setTabOrder(self.pushButtonCancel, self.pushButtonBatch)
        CreateEpochsFromEventsDialog.setTabOrder(self.pushButtonBatch, self.pushButtonApply)
        CreateEpochsFromEventsDialog.setTabOrder(self.pushButtonApply, self.lineEditCollectionName)
        CreateEpochsFromEventsDialog.setTabOrder(self.lineEditCollectionName, self.doubleSpinBoxTmin)
        CreateEpochsFromEventsDialog.setTabOrder(self.doubleSpinBoxTmin, self.doubleSpinBoxTmax)
        CreateEpochsFromEventsDialog.setTabOrder(self.doubleSpinBoxTmax, self.doubleSpinBoxBaselineStart)
        CreateEpochsFromEventsDialog.setTabOrder(self.doubleSpinBoxBaselineStart, self.doubleSpinBoxBaselineEnd)
        CreateEpochsFromEventsDialog.setTabOrder(self.doubleSpinBoxBaselineEnd, self.doubleSpinBoxDelay)
        CreateEpochsFromEventsDialog.setTabOrder(self.doubleSpinBoxDelay, self.checkBoxGrad)
        CreateEpochsFromEventsDialog.setTabOrder(self.checkBoxGrad, self.doubleSpinBoxGradReject)
        CreateEpochsFromEventsDialog.setTabOrder(self.doubleSpinBoxGradReject, self.checkBoxMag)
        CreateEpochsFromEventsDialog.setTabOrder(self.checkBoxMag, self.doubleSpinBoxMagReject)
        CreateEpochsFromEventsDialog.setTabOrder(self.doubleSpinBoxMagReject, self.checkBoxEeg)
        CreateEpochsFromEventsDialog.setTabOrder(self.checkBoxEeg, self.doubleSpinBoxEEGReject)
        CreateEpochsFromEventsDialog.setTabOrder(self.doubleSpinBoxEEGReject, self.spinBoxEventID)
        CreateEpochsFromEventsDialog.setTabOrder(self.spinBoxEventID, self.pushButtonEdit)
        CreateEpochsFromEventsDialog.setTabOrder(self.pushButtonEdit, self.spinBoxMask)
        CreateEpochsFromEventsDialog.setTabOrder(self.spinBoxMask, self.pushButtonHelp)
        CreateEpochsFromEventsDialog.setTabOrder(self.pushButtonHelp, self.listWidgetEvents)
        CreateEpochsFromEventsDialog.setTabOrder(self.listWidgetEvents, self.pushButtonAdd)
        CreateEpochsFromEventsDialog.setTabOrder(self.pushButtonAdd, self.pushButtonClear)
        CreateEpochsFromEventsDialog.setTabOrder(self.pushButtonClear, self.scrollArea)

    def retranslateUi(self, CreateEpochsFromEventsDialog):
        _translate = QtCore.QCoreApplication.translate
        CreateEpochsFromEventsDialog.setWindowTitle(_translate("CreateEpochsFromEventsDialog", "Meggie - Create epochs from events"))
        self.pushButtonCancel.setText(_translate("CreateEpochsFromEventsDialog", "Cancel"))
        self.pushButtonBatch.setText(_translate("CreateEpochsFromEventsDialog", "Batch"))
        self.pushButtonApply.setText(_translate("CreateEpochsFromEventsDialog", "Apply"))
        self.groupBoxEventsList.setTitle(_translate("CreateEpochsFromEventsDialog", "List of event specifications"))
        self.groupBoxRejection.setTitle(_translate("CreateEpochsFromEventsDialog", "Rejection limits"))
        self.checkBoxGrad.setToolTip(_translate("CreateEpochsFromEventsDialog", "Include or exclude grad channels"))
        self.checkBoxGrad.setText(_translate("CreateEpochsFromEventsDialog", "Grad"))
        self.doubleSpinBoxGradReject.setSuffix(_translate("CreateEpochsFromEventsDialog", " fT/cm"))
        self.checkBoxMag.setToolTip(_translate("CreateEpochsFromEventsDialog", "Include or exclude mag channels"))
        self.checkBoxMag.setText(_translate("CreateEpochsFromEventsDialog", "Mag"))
        self.doubleSpinBoxMagReject.setSuffix(_translate("CreateEpochsFromEventsDialog", " fT"))
        self.checkBoxEeg.setToolTip(_translate("CreateEpochsFromEventsDialog", "Include or exclude eeg channels"))
        self.checkBoxEeg.setText(_translate("CreateEpochsFromEventsDialog", "EEG"))
        self.doubleSpinBoxEEGReject.setSuffix(_translate("CreateEpochsFromEventsDialog", " uV"))
        self.groupBoxEpochs.setTitle(_translate("CreateEpochsFromEventsDialog", "Epoch collection"))
        self.labelCollectionName.setText(_translate("CreateEpochsFromEventsDialog", "Collection name:"))
        self.lineEditCollectionName.setText(_translate("CreateEpochsFromEventsDialog", "Epochs"))
        self.labelTmin.setText(_translate("CreateEpochsFromEventsDialog", "Start time:"))
        self.doubleSpinBoxTmin.setSuffix(_translate("CreateEpochsFromEventsDialog", " s"))
        self.labelTmax.setText(_translate("CreateEpochsFromEventsDialog", "End time:"))
        self.doubleSpinBoxTmax.setSuffix(_translate("CreateEpochsFromEventsDialog", " s"))
        self.labelBaselineStart.setText(_translate("CreateEpochsFromEventsDialog", "Baseline start:"))
        self.doubleSpinBoxBaselineStart.setSuffix(_translate("CreateEpochsFromEventsDialog", "s"))
        self.labelBaselineEnd.setText(_translate("CreateEpochsFromEventsDialog", "Baseline end:"))
        self.doubleSpinBoxBaselineEnd.setSuffix(_translate("CreateEpochsFromEventsDialog", "s"))
        self.labelDelay.setText(_translate("CreateEpochsFromEventsDialog", "Delay:"))
        self.doubleSpinBoxDelay.setSuffix(_translate("CreateEpochsFromEventsDialog", "s"))
        self.groupBoxBatching.setTitle(_translate("CreateEpochsFromEventsDialog", "Batching"))
        self.groupBoxEvent.setTitle(_translate("CreateEpochsFromEventsDialog", "Select events to include in epoch collection"))
        self.labelMask.setText(_translate("CreateEpochsFromEventsDialog", "Mask:"))
        self.labelEventID.setText(_translate("CreateEpochsFromEventsDialog", "Event ID:"))
        self.pushButtonEdit.setText(_translate("CreateEpochsFromEventsDialog", "Edit..."))
        self.pushButtonHelp.setText(_translate("CreateEpochsFromEventsDialog", "Help..."))
        self.pushButtonAdd.setText(_translate("CreateEpochsFromEventsDialog", "Add to list >>"))
        self.pushButtonClear.setText(_translate("CreateEpochsFromEventsDialog", "Clear list <<"))

